// ExternControl.h
// ---------------
//
// This code allows up to 4 other CAN enabled controllers (engine ECU, DBW, ABS, turbo controller, etc.) to 
// modify the transmisson's shift algorithm. The modifications are sent over CAN to specific addresses in the in2ram structure.
//
// The user can use MShift to set:
// - each controller's priority relative to the other controllers (bit 0:2 of in2ram.ecXpriority)
// - each controller's assignment (CAN ID to one of the 4 available slots)
// - when other ocntroller can change gears
//
// The external controller can set:
// - whether the requested change is relative (shift up, shift down) or absolute (shift to a specific gear)
// - how minimum time period the gear modification should last (time (seconds), until mode is changed, 
//   until gear is changed (if relative or rpm limits exceeded, only if cancelled by external controller)
//
// In in2ram settings (for MShift user to specify):
//
//    ec1priority = bits,     U08,    abc, [0:2], "Not used", "Highest Priority", "2nd Priority", "3rd Priority", "Lowest Priority"
//    ec2priority = bits,     U08,    abc, [3:5], "Not used", "Highest Priority", "2nd Priority", "3rd Priority", "Lowest Priority"
//    ec3priority = bits,     U08,  abc+1, [0:2], "Not used", "Highest Priority", "2nd Priority", "3rd Priority", "Lowest Priority"
//    ec4priority = bits,     U08,  abc+1, [3:5], "Not used", "Highest Priority", "2nd Priority", "3rd Priority", "Lowest Priority"
//
//    In the case of priority ties, the order will be taken from ec1, ec2, ec3, ec4
//
//    ec1assign   = scalar,   U08,  abc+2, "device 0", 1.0000, 0.0000, 0, 255, 0, 0; Maps CAN_ID to offset for device 0 (engine controller)
//    ec2assign   = scalar,   U08,  abc+3, "device 1", 1.0000, 0.0000, 0, 255, 0, 0; Maps CAN_ID to offset for device 1 
//    ec3assign   = scalar,   U08,  abc+4, "device 2", 1.0000, 0.0000, 0, 255, 0, 0; Maps CAN_ID to offset for device 2 
//    ec4assign   = scalar,   U08,  abc+5, "device 3", 1.0000, 0.0000, 0, 255, 0, 0; Maps CAN_ID to offset for device 3 
//
//    User can set scope of shift changes:
//
//    ec1scope    = bits,     U08,  abc+6, [0:2], "Allow all changes", "Allow changes in auto mode only", "Allow changes in manual mode only", "Switch to auto mode on change", "Switch to manual mode on change", "INVALID", "INVALID", "INVALID"
//    ec2scope    = bits,     U08,  abc+6, [3:5], "Allow all changes", "Allow changes in auto mode only", "Allow changes in manual mode only", "Switch to auto mode on change", "Switch to manual mode on change", "INVALID", "INVALID", "INVALID"
//    ec3scope    = bits,     U08,  abc+7, [0:2], "Allow all changes", "Allow changes in auto mode only", "Allow changes in manual mode only", "Switch to auto mode on change", "Switch to manual mode on change", "INVALID", "INVALID", "INVALID"
//    ec4scope    = bits,     U08,  abc+7, [3:5], "Allow all changes", "Allow changes in auto mode only", "Allow changes in manual mode only", "Switch to auto mode on change", "Switch to manual mode on change", "INVALID", "INVALID", "INVALID"
//
// in2ram passed parameters (from external controller over CAN):
//
//    ec1AbsRel   = bits,     U08,   xxx, [0:0], "Absolute", "Relative"
//    ec1Persist  = bits,     U08,   xxx, [1:2], "Timer", "Until Shift Mode Change", "Until MShift Gear Change", "Until Cancelled"
//    ec1Time     = bits,     U08,   xxx, [3:7], "0.5 second", "1 seconds", "2 seconds", "3 seconds", "4 seconds", "5 seconds", "7 seconds", "10 seconds", "15 seconds", "20 seconds", "25 seconds", "30 seconds", "40 seconds", "50 seconds", "60 seconds"
//    ec1ShiftMod = scalar,   S08, xxx+1,  "", 1.0000, 0.0000, -8, 8, 0, 0; absolute target gear (+only) or relative shift (+/-)
//
//    ec2AbsRel   = bits,     U08, xxx+2, [0:0], "Absolute", "Relative"
//    ec2Persist  = bits,     U08, xxx+2, [1:2], "Timer", "Until Shift Mode Change", "Until MShift Gear Change", "Until Cancelled"
//    ec2Time     = bits,     U08, xxx+2, [3:7], "0.5 second", "1 seconds", "2 seconds", "3 seconds", "4 seconds", "5 seconds", "7 seconds", "10 seconds", "15 seconds", "20 seconds", "25 seconds", "30 seconds", "40 seconds", "50 seconds", "60 seconds"
//    ec2ShiftMod = scalar,   S08, xxx+3, "", 1.0000, 0.0000, -8, 8, 0, 0; absolute target gear (+only) or relative shift (+/-)
//
//    ec3AbsRel   = bits,     U08, xxx+4, [0:0], "Absolute", "Relative"
//    ec3Persist  = bits,     U08, xxx+4, [1:2], "Timer", "Until Shift Mode Change", "Until MShift Gear Change", "Until Cancelled"
//    ec3Time     = bits,     U08, xxx+4, [3:7], "0.5 second", "1 seconds", "2 seconds", "3 seconds", "4 seconds", "5 seconds", "7 seconds", "10 seconds", "15 seconds", "20 seconds", "25 seconds", "30 seconds", "40 seconds", "50 seconds", "60 seconds"
//    ec3ShiftMod = scalar,   S08, xxx+5, "", 1.0000, 0.0000, -8, 8, 0, 0; absolute target gear (+only) or relative shift (+/-)
//
//    ec4AbsRel   = bits,     U08, xxx+6, [0:0], "Absolute", "Relative"
//    ec4Persist  = bits,     U08, xxx+6, [1:2], "Timer", "Until Shift Mode Change", "Until MShift Gear Change", "Until Cancelled"
//    ec4Time     = bits,     U08, xxx+6, [3:7], "0.5 second", "1 seconds", "2 seconds", "3 seconds", "4 seconds", "5 seconds", "7 seconds", "10 seconds", "15 seconds", "20 seconds", "25 seconds", "30 seconds", "40 seconds", "50 seconds", "60 seconds"
//    ec4ShiftMod = scalar,   S08, xxx+7, "", 1.0000, 0.0000, -8, 8, 0, 0; absolute target gear (+only) or relative shift (+/-)
//
// Offset to write received values (2 bytes) to is table 4, offset (xxx + (ecXassign*2)) where ecXassign is the input value dervied from the sending CAN devices ID
//
// Values received are checked for:
// if absolute and ecXShiftMod > NUM_GEARS or ecXShiftMod < 0 then reject change
// if relative, and (current_gear + ec4XhiftMod) > NUM_GEARS or (current_gear + ec4XhiftMod) < 0 then reject change
// if ecXPersist == 'until cancelled' and ecXAbsRel == ('Relative' or 'Timer') then reject change
//
// The external controller can also check the values and MShift status by grabbing the relevant outpc variables in an outmsg, of course.
//
// The mechanism for changing the gear behavior is to alter the target gear (outpc.target_gear),
// so rev checking, 'gear in range' checking, etc. still applies.
// 