// InitProcessor.c
// set up processor (I/O, etc.)
// Could be made into a function InitProcessor() if we wanted to initialize 
// from within main loop.
// i.e. outpc.dbug2 += InitProcessor();
// where function returns 1 if a reset occurs, 0 otherwise  

// Set up COP timeout for main loop
// COPCTL = 0x44;      // 01000100 = 2^20 OSCCLK cycles = 131 ms timeout for reboot
// COPCTL = 0x42;      // 01000010 = 2^16 OSCCLK cycles = 131 ms timeout for reboot
	
PPAGE = 0x3C;  	// PPAGE register is defined in hcs12def.h 
		  		// as memory location 0x0030

/* In order to access the 16K flash blocks in the address range 0x8000 to 0xBFFF 
	the PPAGE register (0X0030) must be loaded with the corresponding value for 
	this range:

	PAGE    PAGE Visible with PPAGE Contents
	----    --------------------------------    
	0x3C    $3C
	0x3D    $3D
	0x3E    $3E
	0x3F    $3F

	For the MC9S12C64, the flash page 3F is also visible in the 0xC000 to 0xFFFF range 
	if ROMON is set. 
	For the MC9S12C64, the flash page 3E is also visible in the 0x4000 to 0x7FFF range
	if ROMHM is cleared and ROMON is set. 
	For the MC9S12C64, the flash page 3D is also visible in the 0x0000 to 0x3FFF range 
	if ROMON is set...  
	
	The main code is responsible for switching PPAGE as needed to select the 
	desired 16K block of memory to be mapped into the "window" from $8000-$BFFF.
	*/
	
restart = 1; //set reboot flag  

//  Initalize PLL (Phase Locked Loop) - reset default 
//  is Oscillator clock 8 MHz oscillator, 
//  PLL freq = 48 MHz, 24 MHz bus, 
//  divide by 16 for timer of 2/3 usec tic
PLLCTL &= 0xBF;     // Turn off PLL so can change freq
					// A &= B is equivalent to A  = A & B, where & means a bit-wise compare
					// Note that this is different from A = &B, which assigns the address 
					// of B to the variable A
SYNR    = 0x02;     // set PLL/ Bus freq to 48/ 24 MHz
REFDV   = 0x00;
PLLCTL |= 0x40;     // Turn on PLL. A |= B is the same as A = A | B 

// wait for PLL lock
while (!(CRGFLG & 0x08)); // Note that & is a bitwise operator, && is the relational operator
CLKSEL = 0x80;      // select PLL as clock

// wait for clock transition to finish
for (ix = 0; ix < 60; ix++);

// Copy Flash to RAM
// open flash programming capability
Flash_Init(8000);
// inp_spare used to force inpflash, flashve_table into sectors.
// Must use in program or it won't use up the entire sector. Codewarrior 
// ignores the pragma making this statement necessary.
ix = in1padding[0];
ix = in2padding[0];
ix = in3padding[0];
	
if((int)RamBurnPgm & 0x0001)	{                    // odd address - copy to even one
	(void)memcpy((void *)RamBurnPgm,NoOp,1);         // copy noop to 1st location
	(void)memcpy((void *)&RamBurnPgm[1],SpSub,32);   // copy flashburn core program to RAM
}  // End if((int)RamBurnPgm & 0x0001)
else                                               // even address
	(void)memcpy((void *)RamBurnPgm,SpSub,32);       // copy flashburn core program to RAM

// load all user inputs from Flash to RAM
// indices are from 'const tableDescriptor tables[NUM_TABLES]' and start at 0
tableInit(4);
tableInit(5);
tableInit(8);

/** Note: ***************************************************  
serial Tx =  PS1
serial Rx =  PS0
CAN    Tx =  PM1
CAN    Rx =  PM0
**************************************************************/ 

//  Set pointers to real port addresses
for(ix = 0; ix < 8; ix++)  {
	pPTMpin[ix] = pPTM;   // port M
	pPTTpin[ix] = pPTT;   // port T
} // end for (ix = 0...

#include "timer_ports.h"

/*
Port DDR (data direction register) Configuration
------------------------------------------------
0=input, 1=output, x=not assigned (default is input)

For the GPIO, we want:
Bit	  76543210     decimal   hexadecimal
	--------     -------   -----------
PA    xxxxxxx1	   = 01      = 0x0001
AD	  00xx0000	   = 00      = 0x0000
PB	  xxx1xxxx	   = 16      = 0x0010
PE	  xxx1xxx1	   = 16      = 0x0011
PM	  xx1111xx	   = 60      = 0x003C
PT	  10011110	   =158      = 0x009E
*/

// Set up the data direction ports and pull-ups

// ** MShift I/O Port Data Direction Register Settings ***
//
// These set the pport pins as inputs (0) or outputs (1).
// Default is input.
//
	//************************************************************
	
DDRA |= 0x01;    // port A0 - output; 0x01 = 00000001 (A0 is a spare output)
DDRB |= 0x10;    // port B4 - output; 0x10 = 00010000  
DDRE  = 0x10;    // port E4 - output; 0x10 = 00010000 (E0 is a input only!), port B4 - output; 0x10 = 00010000, Port E all inputs, except PE4 which is output (solA)
PUCR  = 0x00;    // disable internal pull-ups
INTCR = 0x00;    // disable interrupt on PE1  

DDRJ &= 0x3F;    // port J pins 0 to 5 outputs,             0x3F =  63 = 00111111
PERJ |= 0xC0;    // enable pullup resistance for port J6,7; 0xC0 = 192 = 11000000 

DDRM |= 0x3C;    // port M2, M3, M4, M5 are  outputs, full drive by default; 0x3C = 00111100 

DDRP  = 0x00;    // port P all inputs
PERP  = 0xFF;    // enable pullup resistance for port P;    0xFF = 255 = 11111111

DDRS &= 0xF3;    // port S all outputs, except 2,3;         0xF3 = 243 = 11110011
PERS |= 0x0C;    // enable pullup resistance for port S2,3; 0x0C =  12 = 00001100

DDRT |= 0x9E;    // port T1, T2, T3, T4, T5 & T7 are outputs, T0, T5, & T6 are inputs; 0x9E = 10011110

// turn off output pins
Output2state = 1;               // Output1state, Output2state = 1 -> P,R,N,1
Output1state = 1;              // Output1state, Output2state = 1 -> P,R,N,1
OUTPUT1port &= ~OUTPUT1pin;   // 0x10 = 00010000, ~0x10 = 11101111, turn off    Sol A @ PE4
OUTPUT2port &= ~OUTPUT2pin;   // 0x04 = 00000100, ~0x04 = 11111011, turn off    Sol B @ PM2
OUTPUT3pwm = OUTPUT3_OFF;

LED1_OUT &= ~LED1pin;     // 0x10 = 00010000, ~0x10 = 11101111, turn off     LED1 @ PM4
LED3_OUT &= ~LED3pin;     // 0x20 = 00100000, ~0x20 = 11011111, turn off     LED3 @ PM5
LED2_OUT &= ~LED2pin;     // 0x08 = 00001000, ~0x08 = 11110111, turn off     LED2 @ PM3
LED4_OUT &= ~LED4pin;     // 0x10 = 00010000, ~0x10 = 11101111, turn off     LED4 @ PB4

SPARE1_OUTport &= ~SPARE1pin; // 0x01 = 10000000, ~0x80 = 01111111, turn off   Spare1 @ PT7 
SPARE2_OUTport &= ~SPARE2pin; // 0x01 = 00000001, ~0x01 = 11111110, turn off   Spare2 @ PA0

reinit_flag = 0;

// Set Up CRG RTI Interrupt for .128 ms clock. CRG from 8MHz oscillator.
mms        = 0;     // 0.128 ms tics
millisec   = 0;     // 1.024 ms clock (8 tics) for adcs
lmms       = 0;
cansendclk = 7812;
ltch_lmms  = 0;
can_mmsclk = 0;     // .128 ms tics  - new CAN Comms
outpc.seconds = 0;  // (1.0035) seconds
burn_flag  = 0;
ic1_count  = 1;
ic1_recount= 1;
ic2_count  = 1;

RTICTL   = 0x10;   // load timeout register for .128 ms (smallest possible)
CRGINT  |= 0x80;   // enable interrupt
CRGFLG   = 0x80;   // clear interrupt flag (0 writes have no effect)

// ------ Set up SCI Serial Communications Interface (RS232) ----------------
/* To use SCI0 for 8 bit, non-parity communication:

	1. Select the baud rate via SCI0BDL Baud Rate Register, at memory 
		location 0x00C9 - defined in hcs12def.h (set below). 
		SCI baud rate = SCI module clock / (16 x BR) =
					= 24,000,000/ (16*baud)
					= 1,500,000/baud
					= for baud == 115200 this is 13.0  

	2. Enable transmission and/or reception as desired by setting the 
		TIE (Transmitter Interrupt Enable) and RIE (Receiver Full 
		Interrupt Enable Bit) bits in the SCI0CR2 Control Register,
		which is at memory location 0x00CB. 
		This is register is set up as:
		
		bit        7       6      5      4      3     2     1     0
				TIE    TCIE    RIE    ILIE    TE    RE   RWU   SBK

	3. Tramit and Receive, in interrupt service routine Serial_Comm_ISR():

		- transmit  - poll the TDRE flag and write data to the SCI0DRL register at 
						address $00D7 when the TDRE flag is set. The data we write 
						to the SCI0DRL register (0x00CF) is the data that will be 
						immediately output from the TX pin. 
		- receive   - poll the RDRF register (SCI0SR1). When the RDRF status 
						register (0x00CC) is set ((SCI0SR1 & 0x20) <> 0), we can then 
						read the incoming data by reading the SCI1DRL register. The 
						data we read in from this register is the input via the RX pin. 
*/

// Set the baud rate
if ((inpram.baud < MIN_BAUD) || (inpram.baud > MAX_BAUD))  // if value unreasonable	
	{						
    // if value is very high or very low, for example
    // if all ones in unsigned 32 bit integer from aborted flash burn
    SCI0BDL = 13; // 115200 baud
    inpram.baud = 115200; // reset the flashed value
    outpc.miscStatus |= 0x40; // set 6th (baud error) bit for datalog
	}
    else
    {
	SCI0BDL = (unsigned char)(1500000/inpram.baud);   		// = 1500000/115200 = 13
	ltmp = (150000000/inpram.baud) - ((long)SCI0BDL*100); 	// round to nearest 100
	if(ltmp > 50) SCI0BDL++;   // round up
    outpc.miscStatus &= ~0x40; // clear 6th (baud error) bit for datalog
    }
    	
SCI0CR1 = 0x00;   // = 00000000 -> parity even -- 0000000x 
					//               parity disabled
					//               idle char starts after start bit
					//               idle line wakeup
					//               one start bit, eight data, one stop
					//               only meaningful if LOOPS =1
					//               SCI enabled in wait mode
					//               LOOP disabled -- x0000000
					
SCI0CR2 = 0x24;   // = 00100100 -> TIE = 0                            -- x0000000, 
					//               RIE = 1,(enable receive interrupt) -- 00x00000,
					//               TE  = 0 (Transmitter Enable Bit)   -- 0000x000, 
					//               RE  = 1 (Receiver Enable Bit)      -- 00000x00,
					//  (these will be reconfigured in Serial_Comm_ISR()) 
txcnt     = 0;
rxcnt     = 0;
txmode    = 0;    // receive == 0, transmit > 0
txgoal    = 0;
kill_ser  = 0;
// 4.110 changes
#ifdef COMMS_382X
  kFlag = -2;	// kFlag is set to the serCOM table index from Transmit Interrupt in serComm.h
				// When kFlag is set to 0 - 15, it refers to a table number in the table
				// descriptor structure which Eric set up. It tells the ECU which table 
				// TS wants a CRC number for so it can match it with what it has. 
				// kFlag = -2 is the state waiting for a TS request; 
				// kFlag = -1 is the state of processing the TS request 
				// and when done it goes back to -2.
#else
  vfy_flg   = 0;
#endif
rcv_timeout = 0xFFFFFFFF; 

#include "pwm_ports.h"

#include "adc_ports.h"

/* Initialize CAN comms */
can_reset = 0;
can_id = inpram.mycan_id;
can_varoff = 0;  
CanInit();   

// Make IC highest priority interrupt
// On HCS12 processors, you can promote a single interrupt source to 
// have the highest priority by writing the LSB address of the appropriate 
// interrupt vector to the HPRIO register.
// The interrupt priority has meaning only when two or more interrupt requests 
// are being processed at the same time to determine which ISR will be 
// executed first. Once a certain ISR has started to execute, the interrupt 
// priority does NOT mean another interrupt can interrupt the already executing ISR.
// 4.110 changes
#ifdef COMMS_382X
  HPRIO = 0xEE;
#endif

// enable global interrupts
ENABLE_INTERRUPTS

// Restart PWM counters
PWMCNT1 = 0x00;
PWMCNT2 = 0x00;  // clear counter
PWMCNT3 = 0x00;  // clear counter
PWMCNT4 = 0x00;
PWME   |= 0x1E;  // enable PWM 1,2,3 & 4 (0x1E = 00011110)
				// PT1 = SolC, 
				// PT2 = PC,
				// PT3 = TCC - on/off, 
				// PT4 = speedo output
