/*

This is a series of generalized functions intended to be used to get/send data over SPI

To get a value over SPI:
1. Write junk byte to data register to start the serial clock (and set SPIF flag)
2. Wait until the SPIF and SPTEF flags are set (which indicates data transfer
is finished) by reading the SPI status register until both flags are 1
3. Clear SPI flag by reading the SPI data register
4. Select the device (usually by bringing a general purpose I/O line active)
5. Write data byte to SPI data register
6. Wait until the SPIF and SPTEF flags are set (which indicates data transfer
is finished) by reading the SPI status register until both flags are 1
7. Repeat steps 2 to 6 if more data to send
8. Deselect the device (usually by bringing a general purpose I/O line inactive)


To read data from a SPI device:
1. Select the device (usually by bringing a general purpose I/O line active)
2. Write a junk byte to the SPI data register (starts the serial clock)
3. Wait until the SPI flag is set (which indicates data transfer is finished) by
reading the SPI status register until the SPIF bit is 1 (this is also the first
step in clearing the SPIF flag)
4. Read data from the SPI data register (gets data and completes clearing
the SPIF flag)
5. Repeat steps 2, 3 and 4 if more data to receive
6. Deselect the device (usually by bringing a general purpose I/O line inactive)

*/

void SPI_send8bits(char cx)
{
// This function sends a single byte over SPI.
char temp;

while (!(SPI0SR & SPTEF));   // wait until write is permitted
SPI0DR = cx;                 // output byte to SPI
while (!(SPI0SR & SPIF));    // wait until the operation is complete
temp = SPI0DR;               // clear the SPIF flag by reading it

return;
}

void SPI_send16bits(int cx)
{
// This function sends an integer (2 bytes, 16 bits) from the SPI interface.
char count;
char temp[2];                    // temp[0] is MSB, temp[1] is LSB

temp[0] = (char) (cx / 256);     // MSB	 (not right shift because of uncertainty about the sign bit)
temp[1] = (char) (cx % 256);     // LSB

for(count=0; count < 2; count++)  // continue while the byte count is non-zero
    {                     
    SPI_send8bits(temp[count]);   // send a byte
    }

return;                          // return
}

void SPI_sendString(char *ptr)
{
// This function sends a null terminated string over SPI.
while (*ptr)
  {
  SPI_send8bits(*ptr);
  ptr++;
  }
  return;
}

char SPI_getByte(void)
{
// This function reads a byte from the SPI interface.
  while(!(SPI0SR & SPTEF));  // wait until write is permitted
  SPI0DR = 0x00;             // trigger 8 SCK pulses to shift in data
  while(!(SPI0SR & SPIF));   // wait until a byte has been shifted in
  return SPI0DR;             // return the character
}

int SPI_getInt(void)
{
// This function reads a int (2 bytes, 16 bits) from the SPI interface.
char count;
char temp[2];  // temp[0] is MSB, temp[1] is LSB
int result;

for(count=0; count <2; count++)   // continue while the byte count is non-zero
    {                     
    temp[count] = SPI_getByte();  // get a byte and save it in the buffer
    }

result = 256*temp[0] + temp[1];

return result;                   // return
}

void SPI_getString(char *ptr, char count)
{
// This function reads a string from the SPI interface.
  while(count)               // continue while the byte count is non-zero
    {                     
    *ptr++ = SPI_getByte();  // get a byte and save it in the buffer
	count--;
    }
  *ptr = 0;
  return;        // return
}

char setUp_SPI(char device)
{
 char selected;

 // set up SPI comms
 switch (device)
	{
    case 0:
	  // set up 4MHz SPI for MC33800
	  
	  // SCLK
	  // SCLK is idle low for the MC33800
	  // A SCLK pulse count strategy has been implemented on the MC33800 to ensure 
	  // the integrity of SPI communications. SPI messages consisting of 16 SCLK 
	  // pulses and multiples of 8 clock pulses thereafter will be acknowledged. 
	  // SPI messages consisting of other than 16 + multiples of 8 SCLK pulses will 
	  // be ignored by the device.
	  // Since the HCS12 SPI interface is not able to send 16bit at ones, we 
      // send the first eight bits then the next eight bits by holding CS active.
      
      // SLAVE SELECT
      // CS chip select on the MC33800 is active low
      // Each SPI device must have a slave select line that is asserted (pulled low) before 
      // the clock starts, and that reurn to high at the end of the transmission. 
      
      // We will use:
      // ... to select device 0 (MC33800).
      // PM3 (SS) to select device 1 (4 digit seven segment LED display).
      // ... to select device 2
      // ... to select device 3
	  
	  // SERIAL INPUT DATA (SI)
      // The SI pin is used for serial instruction data input. SI information is 
      // latched into the input register on the rising edge of SCLK. A logic high 
      // state present on SI will program a one in the command word on the rising 
      // edge of the CS signal. To program a complete word, 16-bits of information 
      // must be entered into the device.

      // SERIAL OUTPUT DATA (SO)
      // The SO pin is the output from the shift register. The SO pin remains 
      // tri-stated until the CS pin transitions to a logic low state. All normal 
      // operating drivers are reported as zero, all faulted drivers are reported as 
      // one. The negative transition of CS enables the SO driver.
      
      // The SI/SO shifting of the data follows a first-in-first-out protocol, with 
      // both input and output words transferring the most significant bit (MSB) first.
	  
	  // SPICR1
	  // bit 0: LSBFE    = 0
	  // bit 1: SSOE     = 0  (turn off slave select output enable)
	  // bit 2: CPHA     = 0
	  // bit 3: CPOL     = 0
	  // bit 4: MSTR     = 1  (Master mode)
	  // bit 5: SPTIE    = 0
	  // bit 6: SPE	     = 1  (SPI enabled, port pins are dedicated to SPI functions.)
	  // bit 7: SPIE     = 0
	  // == 01010000 ==  0x50 = 80
	  
	  // SPICR2
	  // bit 0: SPC0     = 0
	  // bit 1: SPISWAI	 = 1 (SPI clock operates normally in wait mode)
	  // bit 2: 0		 = 0
	  // bit 3: BIDIROE	 = 0
	  // bit 4: MODFEN	 = 0 (Do not use dedicated SS pin, use as GPIO instead - for multiple devices)
	  // bit 5: 0		 = 0
	  // bit 6: 0		 = 0
	  // bit 7: 0		 = 0
	  // == 00000010 == 0x02 = 2
 
      SPI0BR  = in3ram.D0BR;   // set the baud rate to 2 MHz  (SPPR = 010 and SPR = 000)
                               // baud = 24Mhz/[(SPPR + 1) * (2^(SPR+1))]
                               //      = 24/(3 * (2^1)) = 24/6 = 4 MHz
                        
      SPI0CR1 = in3ram.D0CR1;  // ==  01010000
                               // disable the SPI interrupt, enable SPI,
                               // SCK idle low, shift data on rising edge, 
                               // shift data MSB first, CPOL=0, slave select output enabled
                        
      SPI0CR2 = in3ram.D0CR2;  // == 00010010
                               // disable bi-directional mode, disable SPI in wait mode, set MODFEN
                        
      WOMM = 0x00;             // 0 = output buffers act as push-pull, 1 = output buffers act as open-drain
      selected = 0;
      break;

   case 1:
      // set up SPI comms for 7-segment 4-digit LED array display
      SPI0BR  = in3ram.D1BR;   // set the baud rate to 2 MHz  (SPPR = 010 and SPR = 000)
                               // baud = 24Mhz/[(SPPR + 1) * (2^(SPR+1))]
                               //      = 24/(3*(2^1)) = 24/6 = 4 MHz
      SPI0CR1 = in3ram.D1CR1;  // disable the SPI interrupt, enable SPI,
                               // SCK idle low, shift data on rising edge, 
                               // shift data MSB first
      SPI0CR2 = in3ram.D1CR2;  // disable bi-directional mode, disable SPI in wait mode
      WOMM = 0x00;             // 0 = output buffers act as push-pull, 1 = output buffers act as open-drain
      selected = 1;
      break;

   case 2:
      // set up SPI comms for device 2
      SPI0BR  = in3ram.D2BR;   // set the baud rate to 12 MHz  (SPPR0 = 0 and SPR0 = 0)
      SPI0CR1 = in3ram.D2CR1;  // disable the SPI interrupt, enable SPI,
                               // SCK idle low, shift data on rising edge, 
                               // shift data MSB first
      SPI0CR2 = in3ram.D2CR2;  // disable bi-directional mode, disable SPI in wait mode
      WOMM = 0x00;             // 0 = output buffers act as push-pull, 1 = output buffers act as open-drain
      selected = 1;
      break;
      
   case 3:
      // set up SPI comms for device 3
      SPI0BR  = in3ram.D3BR;   // set the baud rate to 12 MHz  (SPPR0 = 0 and SPR0 = 0)
      SPI0CR1 = in3ram.D3CR1;  // disable the SPI interrupt, enable SPI,
                               // SCK idle low, shift data on rising edge, 
                               // shift data MSB first
      SPI0CR2 = in3ram.D3CR2;  // disable bi-directional mode, disable SPI in wait mode
      WOMM = 0x00;             // 0 = output buffers act as push-pull, 1 = output buffers act as open-drain
      selected = 1;
      break;
    
   default:
      // no valid device selected
      selected = 99;
      break;  
	}
 return selected;
}

unsigned int calcSPIpwm(unsigned int Hz, char PWMpin, unsigned char PWMduty)
{
// This function takes the PWM parameters (Hertz, pin number and duty cycle as 
// arguements, and returns the appropriate 16 bit value to send to the MC33800.
char control;
char freq;
unsigned int value;

// Calculate control address value for bits 11-15
if      ( PWMpin == 1 ) control = 0x0C;
else if ( PWMpin == 2 ) control = 0x0D;
else if ( PWMpin == 3 ) control = 0x0E;
else if ( PWMpin == 4 ) control = 0x0F;
else if ( PWMpin == 5 ) control = 0x10;
else if ( PWMpin == 6 ) control = 0x11;

// Calculate bits 0-6 for duty cycle select value
// The duty cycle of the PWM outputs is controlled by bits 0 through 6, inclusive. 
// The duty cycle value is 1% per binary count from 1 to 100 with counts of 101 
// through 127 defaulting to 100%.
if (PWMduty > 100) PWMduty = 100;


// Calculate bits 7-8-9 for frequency select value
// Frequency X is used if the specified value is more then 1.5 times the next lower vlaue  
if      ( Hz >= 960 )  freq = 0x07; // 00000111	 = 1280 Hz
else if ( Hz >= 480 )  freq = 0x06; // 00000110	 =  640 Hz
else if ( Hz >= 240 )  freq = 0x05; // 00000101	 =  320 Hz
else if ( Hz >= 120 )  freq = 0x04; // 00000100	 =  160 Hz
else if ( Hz >=  60 )  freq = 0x03; // 00000011	 =   80 Hz
else if ( Hz >=  30 )  freq = 0x02; // 00000010	 =   40 Hz
else if ( Hz >=  15 )  freq = 0x01; // 00000001	 =   20 Hz
else                   freq = 0x00; // 00000000	 =   10 Hz

value = (control * 0x800) + (freq * 0x080) + PWMduty;

return value;	
}