// --- Set Up ADC Channels ---
  // Set up analog-digital converter (ADC) channels so they continuously convert, 
  // we will read one value in the ISR_timer_clock() routine every 25 milliseconds.
  
  /* USING THE HCS12 A/D CONVERTER 
    (* indicates the values we use for MShift)
     
     1. Set up which pins are analog/digital converters and which are digital
        inputs using the ATD0DIEN register, 1=digital input, 0=ADC
     
     2. Power up A/D Converter (ADPU = 1 in ATD0CTL2)

     3. Select number of conversions per sequence (S8C S4C S2C S1C in ATD0CTL3)
          S8C S4C S2C S1C = 0001 to 0111 for 1 to 7 conversions
          S8C S4C S2C S1C = 0000 or 1xxx for 8 conversions

     4. Set up resolution in ATD0CTL4 position 7 (0-7)
         - For  8-bit mode ( 255 divisions) write 1 to ATD0CTL4 position 7
         - For 10-bit mode (1024 divisions) write 0 to ATD0CTL4 position 7
           
     5. Select DJM in ATD0CTL5 position 7
        (a) DJM = 0 => Left justified data in the result registers
        (b) DJM = 1 => Right justified data in the result registers *

     6. Select DSGN in ATD0CTL5 position 6
        (a) DSGN = 0 => Unsigned data representation in the result register *
        (b) DSGN = 1 => Signed data representation in the result register
        The Available Result Data Formats are shown in the following table:
        
        SRES8 DJM DSGN   Data Format
        1      0    0    8-bit/left justified/unsigned   - Bits 15-8
        1      0    1    8-bit/left justified/signed     - Bits 15-8
        1      1    X    8-bit/right justified/unsigned  - Bits  7-0
        0      0    0    10-bit/left justified/unsigned  - Bits 15-6
        0      0    1    10-bit/left justified/signed    - Bits 15-6
        0      1    X    10-bit/right justified/unsigned - Bits  9-0 *

     7. Select SCAN in ATD0CTL5 position 5:
         - SCAN = 0: Convert one sequence, then stop
         - SCAN = 1: Convert continuously *

     8. Select MULT in ATD0CTL5 position 4:
         - MULT = 0: Convert one channel eight the specified number of times
         - Choose channel to convert with CC, CB, CA of ATD0CTL5.
         - MULT = 1: Convert across several channels. CC, CB, CA of ATD0CTL5
           is the first channel to be converted. *

     9. After writing to ATD0CTL5, the A/D converter starts, and the SCF bit
        is cleared. After a sequence of conversions is completed, the SCF flag in
        ATD0STAT0 is set. You can read the results in ATD0DRx [0-7]H.

     10. If SCAN = 0, you need to write to ATD0CTL5 to start a new sequence. If
         SCAN = 1, the conversions continue automatically, and you can read new
         values in ADR[0-7]H.

     11. To get an interrupt after the sequence of conversions are completed, set
         ASCIE bit of ATD0CTL2. After the sequence of conversions, the ASCIF bit
         in ATD0CTL2 will be set, and an interrupt will be generated.

     12. On HCS12 EVBU, AD0 channels 0 and 1 are used to determine start-up
         program (D-Bug12, EEPROM or bootloader). Do not use AD0 channels
         0 or 1 unless absolutely necessary (you need more than 14 A/D
         channels).

     13. To interpret the result, ATD0DRx = (Vin - VRL) / (VRH - VRL) * 1024
         Normally, VRL = 0 V, and VRH = 5 V, so ATD0DRx = (Vin / 5 V) * 1024
         Example: ATD0DR0 = 351 => Vin = 448/1024 * 5.0 = 1.71 V
  */

/*  We want ADC on AD0 (swA), AD1 (swB), AD2 (temp), AD3 (swC), AD4 (line pressure), 
    and AD5 (non_CAN MAP) only. The rest are digital inputs.
     
switchA					PAD00		GPI1     5
switchB					PAD01		GPI2     6
Temp Sensor			    PAD02		GPI3	30
switchC					PAD03		EGT3    25
line pressure sensor    PAD04       EGT2    27
non-CAN MAP/TPS/MAF     PAD05       EGT1    24
Paddle DOWN			    PAD06       GPI5	 4
Brake Sense			    PAD07		GPI4     3

-- These will be sampled by the get_adc() function, one every 25 milliseconds in 
   the Timer_Clock_ISR() function 
*/
//   - PAD00: switchA
//   - PAD01: switchB  (if not used as shift output)
//   - PAD02: temp sense
//   - PAD03: switchC  (if not used as shift output)
//   - PAD04: line pressure
//   - PAD05: non-CAN MAP
//   - PAD06: paddle downshift
//   - PAD07: brake sense

if (inpram.hardware == 0) // if GPIO
  	{
	ATD0DIEN = 0x00;   	// was 0xC0// 0 = disable digital input buffer to PTADx (use as ADC)
							// 1 = enable the digital input buffer to PTADx (use as digital input)
	DDRAD = 0x00;		// All inputs (was ~0x80)
  	PERAD &= ~0x00;	   	// Disable pull-ups
  	}
 	else // if microTCU
  	{	// AND if digital inputs
  	if (inpram.mlever_mode == 0) 
  		{ // Disable the ADC on PAD00, PAD01, and PAD03
  		ATD0DIEN = 0xCB;	// 1=digital input, 0=ADC
                    		// 0xCB = 11001011
                    		// ADC on temperature sensor, line pressure sensor, optional MAP only
                    		// 3 manual shift lever inputs
                    		// rest are digital inputs
                    		// read digital input pins from PORTAD0 
		DDRAD = 0x00; 		// 0 - input, 1 - output
							// It is not recommended to have the digital buffers 
							// enabled on a pin while simultaneously using it as an analog input.

		// Set Internal Pull-Ups
		// ---------------------
 
		// Input1,2,3, PaddleDOWN are PAD00, PAD01, PAD03, PAD06
		// - PaddleDOWN has pull-up on PCB
		// - Brake is PAD07 (pull-up is external to controller)
  
		// So enable pull-ups on PAD00, PAD01, PAD03 only:
		// if (inpram.stdin_cfg & 0x04) // if using as 4th lever input

		// Pullup Enable Register AD 
		// -------------------------
		// This register configures whether a 
		// pull-up or a pull-down device is activated, if the port is used 
		// as input. This bit has no effect if the port is used as output. 
		// Out of reset no pull device is enabled.
		// It is not possible to enable pull devices when a associated ATD 
		// channel is enabled simultaneously.
		// 0 == Pull-up or pull-down device is disabled.
		// 1 == Either a pull-up or pull-down device is enabled.
	
		// first disable the ADC on PAD00, PAD01, PAD03
		// this is done in adc_ports.h near line 94
		// (remember to change inXstate determination from ADC to 1/0 when hardware=microTCU)
	
  		PERAD |= 0x00; // Disable pulls for GPIO;
  	    
  		// Pullup Polarity Select AD
  		// ------------------------- 
  		// This register selects whether a pull-down or 
  		// a pull-up device is connected to the pin.
		// 0 == A pull-up device is connected to the associated port AD pin, 
		// 		if enabled by the associated bit in register PERAD
		// 		and if the port is used as input.
		// 1 == A pull-down device is connected to the associated port AD pin, 
		//		if enabled by the associated bit in register
		//		PERAD and if the port is used as input.
		
  		PPSAD  = 0x00;   // Set to pull-up rather than pull-down
  		
  		} // end if (inpram.mlever_mode == 0)
  	} // end else if (inpram.hardware == 0)
  	 
  next_adc =    0;	// specifies next ADC channel to be read
  ATD0CTL2 = 0x40;  // leave interrupt disabled, set fast flag clear
  ATD0CTL3 = 0x00;  // do 8 conversions/ sequence
  ATD0CTL4 = 0x67;  // 10-bit resolution, 16 tic conversion (max accuracy),
                    // prescaler divide by 16 => 2/3 us tic x 18 tics
                    // 0x67 = 01100111
  ATD0CTL5 = 0xB0;  // right justified, unsigned, continuous conversion,
                    // sample 8 channels starting with AN0
                    // 0xB0 = 10110000
  ATD0CTL2 |= 0x80; // turn on ADC0
  
  // wait for ADC engine charge-up or P/S ramp-up
  for(ix = 0; ix < 160; ix++)  
  	{
    while(!(ATD0STAT0 >> 7));	    // wait until conversion complete
    ATD0STAT0 = 0x80;
  	}  // end for (ix = 0...
  
  if ((inpram.mlever_mode == 1) && (inpram.spareInputs > 0))
    {
     // if both are outputs
     if (inpram.spareInputs == 2)
       {
       ATD0DIEN |=  0x0A;  // set to be digital inputs (not ADC)
       DDRAD |= 0x0A;      // set pins 1 and 3 to 1 (outputs) = 00001010
       }
     // if only Input 2 (PAD01) is an output
     if (inpram.spareInputs == 1)
       {
       ATD0DIEN = 0x02; // set to be digital inputs (not ADC)
       DDRAD = 0x02;    // = 00000010
       }
    }
    
  if ((inpram.out1_pwm_on) && (inpram.mlever_mode == 1) && (inpram.spareInputs > 0) && ((inpram.PWM_enable & 0x10) || (inpram.PWM_enable & 0x20)))
    {
    // reduce output if PWM'd on ADC output
    RDRAD |= 0x0A;
    }
    else
    {
    // full output
    RDRAD = 0x00;
    }
       
  first_adc = 0;
  adc_lmms = lmms;