/****** INPRAM User Inputs *******************************************/
// User inputs from PC/tuning interface - 1 set in flash first, then copy into RAM
// To access the RAM variables, use the inpram. prefix

typedef struct {
// In C, variables must be 'declared' before they can be used. 
// Their 'variable type' must be given.
//
// This code uses a number of variable types:
//
//   Type             Bits              Range               TunerStudio INI Designation
//   ----             ----              -----               ------------------------
// - char               8            -128 to 127                    S08
// - unsigned char      8               0 to 255                    U08
// .................................................................................
// - int               16          -32768 to 32767                  S16
//   (== short)
// - unsigned int      16               0 to 65535                  U16
//   (== unsigned short int)
// .................................................................................
// - long              32            -2 to 2 - 1                S32
//   (== long int)     
// - unsigned long     32               0 to 2 - 1                U32
//   (== unsigned long int)
// .................................................................................
//
// Variables are declared with a statement like: 
//     data_type variable_name;
//            or
//     data_type variable_name = 16; 
//     (which also initializes the variable)
//
// See the application of this below.
//
// In general, the shortest variable that will do the job
// (spans the range of values needed in any circumstance by the program) 
// uses the least space, leaving more room in memory for 
// other variables and code.
//
// These variable declarations can be prefaced by: 
// - const (doesn't change ever - and can be set as 
//   constant by the compiler)
// - volatile (can be changed externally to the program - by a register result, 
//             for example, and therefore should not be 'optimized' by the compiler)
// 
// In addition to the range of values a variable type can hold, there are issues associated with
// what happens if the range is exceeded. In general, the value 'wraps around'. That is,
// it goes to the next value in the range as if the highest and lowest values were connected.
//
// For example, for an unsigned int, you would have:
// 
// 0,1,2,3,4,.....253,254,255,0,1,2,3....
//
// so adding 1 to 255 = 0, add 2 to 255 = 1, etc. This is often referred to as 'overflow'.
// Similarly, subtracting 1 from 0 is 255, and so on. This is sometimes called 'underflow'.
//
// This is easier to understand if you look at the binary representation that 
// the CPU actually uses:
//
//   255 in binary forum is: 11111111, and this uses 8 bits (short int). Add 1 to it, and you get
//   1+11111111 = 100000000
//
// But when you try to stuff that into the 8 bits of an unsigned int, the most significant 
// bit (MSB - the leftmost) is truncated, so you get 00000000
//
// This applies not just to constants stored in variables, but also to calculations. Suppose 
// you have two variables with values A=10000 and B=20000, and you want to multiply then to get C.
//
// You might expect C = A*B = 10000 * 20000 = 200000000
// However, if C is an unsigned short, then C will equal 20000000 % 255 = 95 
// (% is the modulus, the remainder after dividing the first number by the
// second number as many times as it will go)
// However, if C is an unsigned int, then C will equal 20000000 % 65535 = 11825
// Only if C is a long does it equal the expected value of 20000000.
//
// Note that this wrap-around can also happen in intermediate calculations, so you
// must be aware of this in the order you write assignments if you expect large numbers. 
// For example setting: 
// C = (A/B) * 1000000;
// may not give the same result as: 
// C = (A*1000) / (B*1000); 
// if the intermediate calculations overflow or underflow (and this can depend on the type 
// of A, B, and C).
// (Note that values grouped in parenthesis are calculated first, creating the intermediate 
// results.)
//
// Also be aware that the compiler (and the HSC12 processor) does not recognized 
// 'floating point' numbers (those with non-zero decimal portions), in either 
// constants or intermediate calculations. 
//
// In another example, suppose you want to multiply A by PI=3.1415926... to get B. 
// You cannot write 
// B = A * 3.14159; (since decimal values are not allowed), 
// instead you have to write something like 
// B = (A * 314159)/100000; (and you can see why things *might* overflow!)
//
// Finally, you should not write
// B = A * (314159/100000); since the intermediate value (314159/100000) will be 
// truncated to 3 (no floating point numbers), and the result will be B = A * 3;
// (Note: also see 'casts'.)
//
// The Codewarrior compiler will often produce a "possible loss of data" warning if
// it can determine that a calculation or variable might overflow, but it is not 
// fool-proof by any means.

unsigned char InputCaptureEdge;	// Input capture: 0 = rising edge, 1 falling
unsigned int vss_table1[NUM_MPH];// vehicle speed tables
int LOAD_table1[NUM_LOADS];
int temp_table[NUM_TEMPS];  	// deg x 10 (C or F)
unsigned int gear_table[NUM_GEARS]; // gear ratio * 1000
unsigned char CAN_enabled; 
unsigned char menuDisplay; 		// for TS only
int noTCC_temp;            		// no TCC lock below this temp (to aid warm-up) (x10) 
unsigned int noTCC_load;   		// no TCC lock above this load
unsigned char LUF_taper;   		// not used
unsigned char PulseTol;    		// % tolerance for next input pulse
unsigned int axle_ratio;   		// rear axle ratio * 1000 (i.e. 3.08 = 3080),
unsigned int tire_diam,    		// tire diameter (inches * 100)
         min_dcl_spd;  			// minimum decel speed
unsigned char trans_type,  		// transmission type, 0=4L60E, 1=4R70W
 shift_mode;               		// 0=manual, 1=auto (sequential shift), 2=auto (skip shift), 3 = RPM based
unsigned char LPshiftOptn; 		// If spare port 3; 0=use table, 1=use OFF value, 2=use pressure limit value
unsigned char  error_check;		// 0 = no error checking (stim), 1 = take action on reported errors  
unsigned int spare12;    		// Max engine rpm for downshift
unsigned char pwmper4;     		// speedo pwm period    
unsigned char num_teeth;    	// divide factor for input VSS pulses (teeth/rev)
unsigned int
 TCC_PWM_Pd,			   		// TCC PWM period (usec) - keep between 10-25 KHz (100-40 us)
 PC_PWM_Pd,                		// PC PWM period (usec) - keep between 10-25 KHz (100-40 us)
 OUT3_PWM_Pd;              		// Output3 PWM period (usec)
unsigned char 
 BatFac,                   		// Battery PWM correction factor (% (x10) per volt)
 ms2canID,ms2varBLK,       		// Code specific CAN variables
                           		//  - MS2 CAN id (0), and 
                           		//  - MS2 block number for outpc (7) from tableDescriptor
 PCneutral;                		// PC duty cycle in neutral  
unsigned long baud;        		// baud rate
unsigned char board_type;  		// board type (1-255) of this board;
                           		// type=0, reserved.
                           		// type=1, ECU (MS II)
                           		// type=2, Router board
                           		// type=3, Generic I/O board
                           		// type=4, Transmission Controller, ......
unsigned char mycan_id;    		// can_id (address) of this board (< MAX_CANBOARDS). Always 0 for ECU.
unsigned int can_var_rate; 		// rate at which to get CAN variables from MS-II -
                           		// 8 bytes (4 words) per can_var_rate, 0.128 ms tics
unsigned char auto_table1[NUM_LOADS][NUM_MPH];   // The target gear for each LOAD & MPH
unsigned char pc_table[NUM_LOADS][NUM_MPH];     // The PWM% for PC solenoid by load and mph
unsigned int trans_temp_limit; 	// Apply TCC in all applicable gears above this temp, regardless of load
unsigned int pulse_mile;    	// pulses per mile for speedometer output
unsigned char rpm_check;    	// 0 if no rpm checking (stim or non-CAN), 1 if rpm checking
unsigned char gear_hyst;    	// shift hysteresis, in mph x10
unsigned char TCC_follow_sp1; 	// When is sp1 is in selective clutch mode
								// Also activate sp1 when TCC is initially applied in:
								// bit 0: 1st gear 1=apply, 0=don't apply
								// bit 1: 2nd gear 1=apply, 0=don't apply
								// bit 2: 3rd gear 1=apply, 0=don't apply
								// bit 3: 4th gear 1=apply, 0=don't apply
								// bit 4: 5th gear 1=apply, 0=don't apply
								// bit 5: 6th gear 1=apply, 0=don't apply
								// bit 6: 7th gear 1=apply, 0=don't apply
								// bit 7: 8th gear 1=apply, 0=don't apply
unsigned char max_shift_pressure; // maximum shift line pressure
unsigned int pressure_delay;	// delay for line pressure to change before shift (milliseconds x121)
unsigned char TCC_follow_sp2; 	// When is sp2 is in selective clutch mode
								// Also activate sp2 when TCC is initially applied in:
								// bit 0: 1st gear 1=apply, 0=don't apply
								// bit 1: 2nd gear 1=apply, 0=don't apply
								// bit 2: 3rd gear 1=apply, 0=don't apply
								// bit 3: 4th gear 1=apply, 0=don't apply
								// bit 4: 5th gear 1=apply, 0=don't apply
								// bit 5: 6th gear 1=apply, 0=don't apply
								// bit 6: 7th gear 1=apply, 0=don't apply
								// bit 7: 8th gear 1=apply, 0=don't apply
unsigned char load_adj;     	// bit 0: brake load adjust 0=usual rate, 1=slow rate
								// bit 1: decel load adjust 0=usual load, 1=limit load
								// bit 2: rising load adjust 0=usual rate, 1=rise quickly
								// bit 3: tps at WOT: 0=usual rate, 1=rise quickly
unsigned int LOADshortcount;	// short term load filtering factor
unsigned int LOADlongcount; 	// short term load filtering factor
signed int LOADmult;        	// multiplier for non-CAN voltage to kPa conversion (x100)
signed int LOADzero;        	// y-axis intercept for non-CAN conversion (x100)
unsigned int minTCCspeed;   	// minimum TCC lock speed
signed char PCtemp[NUM_TEMPS]; 	// Temperature adjustments for line pressure (x10)
unsigned int Output1ptrn;     	// Binary output pattern for Output1
unsigned int Output2ptrn;     	// Binary output pattern for Output2
unsigned int Output3ptrn;     	// Binary output pattern for Output3
unsigned int Output4ptrn;     	// Binary output pattern for Output4
unsigned char out3dc;      		// output3 duty cycle when on

unsigned int spspeed[2]; 	  	// spare port speed threshold
unsigned int spspeed_hyst[2]; 	// spare port speed hysteresis
unsigned char spspeed_cond[2];  // spare port speed operator
								// 00 = no condition, 01 = greater than, 10 = less than
unsigned int sprpm[2]; 
unsigned int sprpm_hyst[2];   	// spare port rpm condition
unsigned char sprpm_cond[2];

unsigned int spload[2]; 
unsigned int spload_hyst[2];  	// spare port load conditions
unsigned char spload_cond[2];

signed char spgear[2]; 
unsigned char spgear_cond[2];  	// spare port gear conditions

unsigned int Inputptrn[4];      // Binary output pattern for inputs
unsigned char forceAUTOoptn;  	// Options to force auto mode if over/under rev  
                              	// 1 = force auto if shift_mode is manual and gear < 1
                              	//     or (over-reved or under-reved)
                              	// 2 = force auto mode only if over_reved
                              	// 3 = force auto mode only if under_reved
                              	// 4 = force auto if shift_mode is manual, and gear < 1
                              	// 5 = force auto mode only if over-reved or under_reved
                              	// 6 = force auto mode only if over-reved or under_reved                         
unsigned int mleverVolts[11];    // if mlever_mode=1, voltage in gears
								// P N R 1 2 3 4 5 6 7 8
unsigned char In4_OD;      		// 0 if using lever for fourth, 1 if separate switch (ground for 4th)                           
unsigned char SlipConditions; 	// 0 = No slip adjustments
								// 1 = Adjust spark only
								// 2 = Adjust spark AND line pressure 
								// 3 = Adjust spark THEN line pressure
								// 4 = Adjust line pressure only
								// 5 = Adjust line pressure then spark
unsigned char clutch1up;    	// 1-2, 2-3, 3-4, 4-5, 5-6, 6-7, 7-8; start at right, 0=off, 1=on  
unsigned char clutch2up;    	// 1-2, 2-3, 3-4, 4-5, 5-6, 6-7, 7-8; start at right, 0=off, 1=on
unsigned char clutch1dwn;   	// 2-1, 3-2, 4-3, 5-4, 6-5, 7-6, 8-7; start at right, 0=off, 1=on  
unsigned char clutch2dwn;   	// 2-1, 3-2, 4-3, 5-4, 6-5, 7-6, 8-7; start at right, 0=off, 1=on                                   
unsigned int FWD_factor;   		// speedo adjustment when PE1 is low (in four wheel drive)                                       
signed int upshift_retard;  	// in degrees x10
signed int dwnshift_retard; 	// in degrees x10
signed int gear2_retard;    	// second  gear retard (degx10)
signed int gear3_retard;    	// third gear retard (degx10)
signed int gear4_retard;    	// fourth gear retard (degx10)
unsigned int Tretard_load;     	// timing retard above load (kpax10)
unsigned char LUF_off;         	// Lock-up feel solenoid parameters (%, kPa, %)
unsigned int  LUF_loLoad, LUF_hiLoad; 
unsigned char LUF_loPWM, LUF_hiPWM;
unsigned int digital_threshold; // ADC to digital on/off threshold
unsigned int shftr_accum;       // shifter accumulator - button sensitivity (lower is more sensitive)
unsigned int hyst_enable_speed; // hysteresis disabled below this speed (x10)
unsigned char vss_mask;         // vss input mask (%)
unsigned char stdin_cfg;        // standard inputs configuration
                                // bit 0: 0=just log,        1=use FWD as speedo switcher
                                // bit 1: 0=just log,        1=use as trans temperature
                                // bit 2: 0=report ADC,      1=report temperature (from cltfactor.inc)
                                // bit 3: 0=use as in bit 0, 1=use as 4th channel for gear input
unsigned char LUF_on;           // Lock-up feel ON PWM% value
unsigned char min_speed;        // minimum speed, will set to zero below this (VSS interrupt still active)
unsigned int vss_error_max;     // number of VSS errors before rest
unsigned char line_units;       // line pressure (use table or report ADC or use as engine brake input or use 2-pt calcs)
unsigned int inj_flow;          // injector flow rate (lbs/hour) x10 for ALL injectors for mileage calcs
unsigned char open_time;        // injector open time (msecX100) for mileage calcs
unsigned int fuel_density;      // fuel density (pounds/gallon)

unsigned char nSquirts;         // number of squirt (divide by 2 if alternating)
unsigned int refresh_int;       // the interval time (msec) between dithers for output1 and output2
unsigned int refresh_dur;       // the 100% duty cycle time for output1 and output2
unsigned int dither_intPC;      // the interval time (msec) between dithers for the pressure control output
unsigned int dither_durPC;      // the 100% duty cycle time for the pressure control output
unsigned char out1_pwm_off;     // output1 and output2 off duration (in 0.128 clock tics)
unsigned char out1_pwm_on;      // output1 and output2 on duration (in 0.128 clock tics)
unsigned char brake_ON_polarity;// 1 == high, 0 == low
unsigned int WOT_tps_threshold; // WOT TPS threshold (in ADC counts in code, % of 1024 in MT)
unsigned char iss_mask;         // VSS input mask (%)
unsigned char ic2_usage;        // 0 = 00000000 = not used
                                // 1 = 00000001 = ISS
                                // 2 = 00000010 = tach
                                // 7th (leftmost) bit used for ISS input edge capture:
								// ic2_edge;         
								//    0 = rising
                                //    1 = falling
unsigned char iss_divider;      // ISS/tach divider (number of teeth/rev)
unsigned char activate_xrate;   // 8 bits, one for each CAN outmsg.  When bit m set to 1, message m
                                // will be auto transmitted at rate outmsg[m].xrate.
unsigned char num_gears;        // number of forward gears
signed int gear5_retard;        // fifth gear retard (degx10)
signed int gear6_retard;        // sixth gear retard (degx10)
signed int gear7_retard;        // seventh gear retard (degx10)
signed int gear8_retard;        // eighth gear retard (degx10)
unsigned char Adj_blk;          // Fuel/Spark/Idle block on MS-II
unsigned int Adj_varoffset;     // Fuel/Spark/Idle offset on MS-II
unsigned char LED1, LED2, LED3, LED4; // LED bit patterns
unsigned char LEDN, LEDR;       // LED bit pattern in neutral and reverse
short IACAdjngear;              // IAC adjustment when gear==1 and VSS==0 and TPS==0
short AdvAdjngear;              // Timing  adjustment when gear==1 and VSS==0 and TPS==0
unsigned int idle_tps_adc;      // idle TPS adc count (for idle-up adjustments)
unsigned char can_testout;      // CAN mode to get MS-II outpc variables
                                // 0 = get entire outpc.
                                // 1 = get only outmsg variables
signed int min_manual_speed;    // minimum speed in manual mode before switch to auto mode (can be 0, or negative to disable)                             
unsigned char shft_btn_polarity;// shift button polarity 0=low, 1=high
unsigned char TCC_PWM;          // 0 - on/off
                                // 1 = PWM'd 
unsigned char spareInputs;      // if mlever_mode == voltages
                                // 0 - use as ADCs
                                // 1 - use as shift outputs
                                // 2 - use Input1 as shift output, Input2 as shift solenoid output  
unsigned int Output5ptrn;       // Binary output pattern for output5
unsigned int Output6ptrn;       // Binary output pattern for output6 
unsigned char PWM_enable;       // PWM enable for shift solenoids
unsigned int maxSpeed;          // maximum speed for VSS filtering 
unsigned char resetTime;        // time (seconds x10) for no VSS filtering on reset
unsigned char TCC_hyst;         // seconds (x10) to disable TCC lock-up after unlocking 
unsigned int STCC_rpm;          // minimum rpm to apply full time TCC 
unsigned char STCC_gear;        // minimum gear to apply full time TCC
unsigned int STCC_ULrpm;        // rpm to unlock STCC
unsigned char VSSsmoothFactor;  // smoothing factor for VSS (0-100) 
unsigned char TCC_highGear;     // 0 - use normal limits, 1 - always lock in highest gear
unsigned int TCCupshift;  		// same patterns as Outputs
                          		// to 2nd = 0x10, 3rd = 0x20, 4th = 0x40, 5th = 0x80, 
                          		// 6th = 0x100, 7th = 0x200, 8th = 0x400
                          		// 8 7 6 5 4 3 2 1 R N P 
                          		// 0 0 0 0 1 0 0 1 1 1 1
unsigned char maxUPpressure[8]; // Maximum pressure when shifting up to the specified gear
unsigned char maxDWNpressure[8];// Maximum pressure when shifting down to the specified gear

unsigned char solCupshiftPWM[7];// upshift PWM% for output3/solC/(PWMDTY1/PWMPER1*100%)
                                // bit    0x01 0x02 0x04 0x08 0x10 0x20
				                // shift  1-2  2-3  3-4  4-5  6-7  7-8
unsigned char solCdwnshiftPWM[7];// downshift PWM% for output3/solC/(PWMDTY1/PWMPER1*100%)
                                // bit    0x01 0x02 0x04 0x08 0x10 0x20
				                // shift  2-1  3-2  4-3  5-4  7-6  8-7
unsigned int debounce;     		// Threshold for switch manifold debounce
                           		// Initially set to mid-point,
                           		// incremented/decremented by one each time through loop
                           		// until all switches are <10% or >90% of debounce value
unsigned char WOT_dur;     		// WOT hysteresis blocking duration in seconds
unsigned char mlever_mode; 		// manual level mode:
                           		// 0 for swA,swB, swC  (GM)
                           		// 1 for voltage based (Ford) 
unsigned char shftbut_mode;     // shift button mode 0 = two grounded input, 1 = single ADC, 2 = log only                                
unsigned int no_button_V;       // Volts for neither button pressed (ADC count)
unsigned int up_button_V;       // Volts for sw1 button pressed (ADC count)
unsigned int dwn_button_V;      // Volts for sw2 button pressed  (ADC count)
unsigned int both_button_V;     // Volts for both buttons pressed  (ADC count)
unsigned int tol_button_V;      // Volts for both buttons pressed
unsigned char hardware;         // 0 = GPIO, 1 = MicroTCU
unsigned char sp_mode[2];       // SP1/2 mode; 0 = spare port, 1 = PWM, 2 = TCC, 3 = shift solenoid, 4 = selective clutch
unsigned char sp1_pwm_table[12];// PWM lookup table for SP1 when SP1_mode == 1
unsigned char sp2_pwm_table[12];// PWM lookup table for SP2 when SP2_mode == 1
unsigned int sp1_pwm_indx[12];  // PWM lookup table index for SP1 when SP1_mode == 1
unsigned int sp2_pwm_indx[12];  // PWM lookup table index for SP2 when SP2_mode == 1
unsigned char sp1_index_mode;   // value to compare to sp1_pwm_index
                                // 0 = load, 1 = speed, 2 = rpm, 3 = clt, 4 current gear
unsigned char sp2_index_mode;   // value to compare to sp2_pwm_index
                                // 0 = load, 1 = speed, 2 = rpm, 3 = clt, 4 current gear
unsigned char sp_active[2];     // if SP1,SP2 is PWM, 0 - always on, 1 - on except shifting, 2 on only during shift
unsigned char sp1_off;          // if SP1 is PWM, this is the 'off state' PWM%
unsigned char sp2_off;          // if SP2 is PWM, this is the 'off state' PWM%
unsigned char sp1_pwm_per;      // spare port 1 off duration (in 0.128 ms clock tics)
unsigned char sp2_pwm_per;      // spare port 2 off duration (in 0.128 ms clock tics)
unsigned int Output7ptrn;           // Binary output pattern for output7
unsigned char out2_pwm_off;     // OFF ticks for output 2
unsigned char out2_pwm_on;      // ON ticks for output 2
int minTCC_gear;           		// minimum gear (2-4) for TCC lock  
unsigned char TCC_follow_sp1dur;// duration (0.010 tics) for activation with TCC			
int LOAD_table2[NUM_LOADS];	
unsigned int vss_table2[NUM_MPH];// vehicle speed tables
unsigned char auto_table2[NUM_LOADS][NUM_MPH];   // The target gear for each LOAD & MPH
int LOAD_PCtable[NUM_LOADS];	// vehicle load table for pressure control PWM%
unsigned char LEDP;             // LED pattern in Park
signed int volt1, volt2;		// ADC for point 1 & 2 of pressure sensor
unsigned int press1, press2;    // Pressure for point 1 & 2 of pressure sensor
unsigned char mleverdead;       // plus/minus dead band for manual lever voltage
signed int Sretard_load;        // Minimum load to apply timing retard based on internal slip
signed int Slip_adv;
signed int Slip_fuel;
unsigned char Slip_threshold;   // 
unsigned int INdbug; 	        // debug input value that can be assigned as needed
unsigned char PCreverse;        // Line pressure PWM % in reverse
unsigned char PCOptn;   		// Line pressure option for reverse: 0 = User specified, 1 = Use pressure table
								// bit 0 - Reverse: 0=User specified, 1=Table
								// bit 1 - Neutral: 0=User specified, 1=Table
unsigned char leverErr;   	    // 0 = ignore lever errors, 1 = check lever errors
unsigned int minADCLeverVolts;	// maximum ADC count for ADC lever 
unsigned int maxADCLeverVolts;	// maximum ADC count for ADC lever
unsigned char slipDelay;   	    // time between slip line and slip spark adjustments (10ths) or vice-versa
unsigned char NAGmode;          // 0 = regular mode (current gear from solenoid states); 
								// 1 = NAG mode (current gear from last shift
								// 2 = 4L60e
								// 3 = 4L80e
unsigned char follow_mode;      // 0 - normal auto mode, 1 - follow mode (up to maxgear)
unsigned char follow_maxgear;   // highest gear for burn-out mode
unsigned char brake_load;       // if != 0, update load while braking by brake_load %
unsigned int decel_TPS;         // highest TPS for decel mode
unsigned char VSS_location;     // 0 = Between Trans. and Final Drive (typ. RWD)
								// 1 = Between Final Drive and wheel (typ. FWD) 
unsigned char TCC_follow_sp2dur;// duration (0.010 tics) for activation with TCC 
} inputs1;

/****** IN2RAM User Inputs *******************************************/
// User inputs from PC/tuning interface - 1 set in flash first, then copy into RAM
// To access the RAM variables, use the in2ram. prefix

typedef struct {
unsigned int vss_PCtable[NUM_MPH];    // vehicle speed table for pressure control PWM%				    
signed int speedoCorr[NUM_MPH];       // vehicle speed table for pressure control PWM%				    
unsigned int speedoCorrBin[NUM_MPH];  // vehicle speed table for pressure control PWM%				    

unsigned char pad201;    		// pad byte
unsigned char pad202;    		// pad byte
unsigned char pad203;    		// pad byte
unsigned char pad204;    		// pad byte
unsigned char pad205;    		// pad byte
unsigned char pad206;    		// pad byte
unsigned char pad207;    		// pad byte
unsigned char pad208;    		// pad byte
unsigned char pad209;    		// pad byte
unsigned char pad210;    		// pad byte
unsigned char pad211;    		// pad byte
unsigned char pad212;    		// pad byte
unsigned char pad213;    		// pad byte
unsigned char pad214;    		// pad byte

unsigned char sp0_mode;      	// spare port 0 mode: 0 = speedo output, 1 = PWM, 2 = TCC, 3 = shift solenoid
unsigned char sp0_active;    	// spare port 0 PWM active:  0 - always on, 1 - on except shifting, 2 on only during shift
signed int sp0_16Bin[NUM_MPH]; 	// sp0 16 element bins				    
signed int sp0_9Bin[NUM_LOADS];  	// sp0 9 element bins				    
unsigned char sp0_table[NUM_LOADS][NUM_MPH];   // The target gear for each LOAD & MPH
unsigned int sp0_per;       	// sp0_period (usec)
unsigned char sp0_off;       	// spare port 0 off PWM%
unsigned char sp0_9index_mode;  // value to compare to sp0_9bin
                                // 0 = load, 1 = speed, 2 = rpm, 3 = clt, 4 current gear, 5 = MAP, 6 = TPS
unsigned char sp0_16index_mode; // value to compare to sp0_16bin
                                // 0 = load, 1 = speed, 2 = rpm, 3 = clt, 4 current gear, 5 = MAP, 6 = TPS
unsigned char SPIsoln2;  		// bit 0 = TCC (1=use SPI)
                         		// bit 1 = PC  (1=use SPI)
unsigned char clutch0up;    	// 1-2, 2-3, 3-4, 4-5, 5-6, 6-7, 7-8; start at right, 0=off, 1=on  
unsigned char clutch0dwn;    	// 2-1, 3-2, 4-3, 5-4, 6-5, 7-6, 8-7; start at right, 0=off, 1=on
unsigned int Output8ptrn; 		// Binary output pattern for output8
unsigned char LEDflash;         // LED flash in manual bit0; 0= no, 1=flash 
unsigned char nestINTS;         // bit0 enable nested interrupts (0=no, 1=yes)   
unsigned char SlipOptn;   	    // bit0 slip reporting options (0=TCC slip, 1=internal slip)
unsigned char TPStable[6][6];   // The look-up load for each TPS & RPM
unsigned int TPSrpm[6];			// engine speed bins				    
signed int TPStps[6];			// TPS bins				    
unsigned char TPSmap;			// If not using 6x6 table and using CAN, TPS/MAP% for load value 
signed int shiftCompBins[8];	// bins for shift completion table
signed char shiftCompValues[8]; // values for shift completion table
unsigned char sp3_mode;      	// spare port 3 mode: 0 = line pressure, 1 = PWM, 2 = TCC, 3 = shift solenoid
unsigned char sp3_active;    	// spare port 3 PWM active:  0 - always on, 1 - on except shifting, 2 on only during shift
signed int sp3_16Bin[NUM_MPH]; 	// sp3 16 element bins				    
signed int sp3_9Bin[NUM_LOADS]; // sp3 9 element bins				    
unsigned char sp3_table[NUM_LOADS][NUM_MPH];   // The target gear for each LOAD & MPH
unsigned int sp3_per;       	// sp3_period (usec)
unsigned char sp3_off;       	// spare port 3 off PWM%
unsigned char sp3_9index_mode;  // value to compare to sp3_9bin
                                // 0 = load, 1 = speed, 2 = rpm, 3 = clt, 4 current gear, 5 = MAP, 6 = TPS
unsigned char sp3_16index_mode; // value to compare to sp3_16bin
                                // 0 = load, 1 = speed, 2 = rpm, 3 = clt, 4 current gear, 5 = MAP, 6 = TPS
unsigned int Output9ptrn; 		// Binary output pattern for output9
unsigned char clutch3up;    	// 1-2, 2-3, 3-4, 4-5, 5-6, 6-7, 7-8; start at right, 0=off, 1=on  
unsigned char clutch3dwn;    	// 2-1, 3-2, 4-3, 5-4, 6-5, 7-6, 8-7; start at right, 0=off, 1=on
unsigned char Slip_limit[8];    // Maximum 100-PWM% after slip limit is applied
unsigned char decel_Delay;      // deceleration downshift delay in tenths of a second
unsigned char lever_timer;	  	// manual gear lever error timer (0 if no error, 1 if reporting error, 2..255 if counting down)
unsigned char maxDeltaLever;  	// maximum lever position change
unsigned char unused;	    	// not used
unsigned int msOutpcBytes;		// number of bytes received from MS ECU
unsigned char TCC_hyst;			// TCC hysteresis value
unsigned char speedoutDC;		// speedo PWM % high
unsigned int gear_indic_per[11];  // frequency in gears (0=P,1=N,2=R,3=1st,4=2nd,...10=8th)
unsigned char gear_indic_dc[11];  // duty cycle in gears (0=P,1=N,2=R,3=1st,4=2nd,...10=8th)
unsigned char gearIndctrMode;   // gear indicator mode on spare port 2 
							 	// 0 = current gear
								// 1 = manual gear;
unsigned int ISSratio[9];       // Input Shaft Speed ratio to engine RPM: rpm/ISS
unsigned char burnoutMaxGear;   // maximum gear for burnout mode (follow lever)
unsigned char NAGrpmTol; 		// after shift rpm tolerance in NAG mode (%)
unsigned char NAGout1;			// output1 bit pattern for 1-2/4-5 (y3) solenoid in NAG mode
unsigned char NAGout2;			// output1 bit pattern for 2-3 (y5) solenoid in NAG mode
unsigned char NAGout3;			// output1 bit pattern for 3-4 (y4) solenoid in NAG mode
unsigned int NAGwait;			// NAG shift timeout in milliseconds
unsigned char NAGretard;		// NAG mode use shift retard by bit
unsigned char ignoreLever;     	// 0 - use lever inputs
								// 1 - assume always in highest forward gear
unsigned int decel_Load;		// Decel mode short-term load ceiling
unsigned int inter_ratio;   	// intermediate ratio * 1000 (i.e. 1.131 = 1131),;
unsigned char gear_tolerance;   // percent gear ratio tolerance to declare a found gear
signed int RPMloadDWNbins[8];	// downshift load bins for RPM based shifting
signed int RPMloadUPbins[8];	// upshift load bins for RPM based shifting
unsigned int RPMshiftTable[112];  	// The RPM to shift up/down at for the load bins
									// rows 0 - 6  are the upshift   
									// values for shifts 1/2 ... 7/8 
									// (i.e. current_gear - 1)
									// rows 7 - 13 are the downshift values 
									// for shifts 2/1 ... 8/7 (i.e. current_gear + 5)
unsigned int CANVSSadj;         // CANbus VSS adjustment 

unsigned int canPW1outpc;       // CANbus offset for outpc PW1 (2)
unsigned int canRPMoutpc;       // CANbus offset for outpc RPM (6)
unsigned int canMAPoutpc;       // CANbus offset for outpc MAP (18)
unsigned int canTPSoutpc;       // CANbus offset for outpc TPS (24)
unsigned int canVBATToutpc;     // CANbus offset for outpc vBatt (26)

unsigned char canPW1outmsg;     // CANbus offset for outmsg PW1 (0)
unsigned char canRPMoutmsg;     // CANbus offset for outmsg RPM (2)
unsigned char canMAPoutmsg;     // CANbus offset for outmsg MAP (4)
unsigned char canTPSoutmsg;     // CANbus offset for outmsg TPS (6)
unsigned char canVBATToutmsg;   // CANbus offset for outmsg vBatt (8)
unsigned char canSPEEDOoutmsg;  // CANbus offset for outmsg speedo (-)
								// (outpc equivalent is canVSSoffset)
								
unsigned char VSS_ISR;			// 0 = old VSS ISR,
								// 1 = new VSS ISR
unsigned int canVSSoffset;	  	// offset in outpc for CANbus speed
unsigned char hyst_delay;		// hysteresis timer delay
unsigned int under_rev_limit[7];// lower RPM limits for gears 2 through 8
unsigned int rpm_limit[7];      // upper RPM limits for gears 1 through 7
unsigned int test_mode;		  	// bit 0    (0x01) - 0 = no testing, 1 = testing
								// bit 1    (0x02) - 1 = test Input 1 - use IN1dbug as ADC input for PAD00
								// bit 2    (0x04) - 1 = test Input 2 - use IN2dbug as ADC input for PAD01
								// bit 3    (0x08) - 1 = test Input 3 - use IN3dbug as ADC input for PAD03
								// bit 4    (0x10) - 1 = test temperature - use TMPdbug as ADC input for temperature (PAD02)
								// bit 5    (0x20) - 1 = test VSS - use VSSdbugx100 as period for VSS
								// bit 6    (0x40) - 1 = test ISS - use ISSdbugx100 as period for ISS
								// bit 7    (0x80) - 1 = test Paddle UP - use TSTdbug {bit 0} as ADC input for upshift paddle (PT6)
								// bit 8   (0x100) - 1 = test Paddle DOWN - use PDNdbug as input for downshift paddle (PAD06)
								// bit 9   (0x200) - 1 = test Brake - use BRKdbug as input for brake (PAD07)
								// bit 10  (0x400) - 1 = test line pressure - use LPdbug as ADC input for temperature (PAD04)
								// bit 11  (0x800) - 1 = test 4WD - use TSTdbug {bit 1} as FWD input (PE1)
								// bit 12 (0x1000) - 1 = test non-CANbus Load - use nCANdbug as ADC input
								// bit 13 (0x2000) - 1 = test CANbus Load - use CANloaddbug as CANbus load input
								// bit 14 (0x4000) - 1 = test CANbus RPM - use CANrpmdbug as CANbus RPM input
								// bit 15 (0x8000) - 1 = test CANbus vBatt - use CANbattdbug as CANbus battery voltage input

unsigned char IN1dbug;			// Input 1 ADC count (/4)
unsigned char IN2dbug;			// Input 2 ADC count (/4)
unsigned char IN3dbug;			// Input 3 ADC count (/4)
unsigned char TMPdbug;			// Temperature ADC count (/4)
unsigned char VSSdbug;			// Speed (in MPH)
unsigned char ISSdbug;			// iss/tach
unsigned char PDNdbug;			// Downshift paddle ADC (/4)
unsigned char TSTdbug;          // bit 0 (0x01) - Paddle Up state
								// bit 1 (0x02) - 4WD/Table shift state
unsigned char BRKdbug;			// Brake ADC count (/4)
unsigned char LPdbug;			// Line Pressure ADC count (/4)
unsigned char nCANdbug;			// non-CAN Load ADC count (/4)

signed int CANloaddbug;			// CANbus test load value
unsigned int CANrpmdbug;		// CANbus test RPM value
signed int CANbattdbug;		  	// CANbus test battery voltage value
// 1016 bytes
 } inputs2;
 
/****** IN3RAM User Inputs *******************************************/
// User inputs from PC/tuning interface - 1 set in flash first, then copy into RAM
// To access the RAM variables, use the in3ram. prefix

typedef struct {

unsigned char pad326;
unsigned char pad325;
unsigned char pad324;
unsigned char pad323;
unsigned char pad322;
unsigned char pad321;
unsigned char pad320;
unsigned char pad319;
unsigned char pad318;
unsigned char pad317;
unsigned char pad316;
unsigned char pad315;
unsigned char pad314;
unsigned char pad313;
unsigned char pad312;
unsigned char pad311;

unsigned char SPIsoln1;  // bit 0 = sol A = Output 1 (1=use SPI)
                         // bit 1 = sol B = Output 2 (1=use SPI)
                         // bit 2 = sol C = Output 2 (1=use SPI)
                         // bit 3 = sol D = Output 2 (1=use SPI)
                         // bit 4 = sol E = Output 2 (1=use SPI)
                         // bit 5 = sol F = Output 2 (1=use SPI)
                         // bit 6 = TCC (1=use SPI)
                         // bit 7 = PC  (1=use SPI)
                         
unsigned char D0BR;    // SPI0BR baud rate for device 0
  // set up SPI for MC33800
  // SPICR1
  // bit 0: LSBFE    = 0
  // bit 1: SSOE     = 1  (turn on slave select output enable (requires MODFEN == 1))
  // bit 2: CPHA     = 0
  // bit 3: CPOL     = 0
  // bit 4: MSTR     = 1  (Master mode)
  // bit 5: SPTIE    = 0
  // bit 6: SPE	     = 1  (SPI enabled, port pins are dedicated to SPI functions.)
  // bit 7: SPIE     = 0
unsigned char SPICR2;	  
  // SPICR2
  // bit 0: SPC0     = 0
  // bit 1: SPISWAI	 = 1 (SPI clock operates normally in wait mode)
  // bit 2: 0		 = 0
  // bit 3: BIDIROE	 = 0
  // bit 4: MODFEN	 = 1 (Use SS pin)
  // bit 5: 0		 = 0
  // bit 6: 0		 = 0
  // bit 7: 0		 = 0
unsigned char D0CR1;   // SPI0CR1 for device 0
unsigned char D0CR2;   // SPI0CR2 for device 0
unsigned char D1BR;    // SPI0BR baud rate for device 1
unsigned char D1CR1;   // SPI0CR1 for device 1
unsigned char D1CR2;   // SPI0CR2 for device 1
unsigned char D2BR;    // SPI0BR baud rate for device 2
unsigned char D2CR1;   // SPI0CR1 for device 2
unsigned char D2CR2;   // SPI0CR2 for device 2
unsigned char D3BR;    // SPI0BR baud rate for device 3
unsigned char D3CR1;   // SPI0CR1 for device 3
unsigned char D3CR2;   // SPI0CR2 for device 3
unsigned char spi_enable;  // spi device enable
                             // 0 = disabled, 1 = enabled 
                             // bit 0 = device zero (MC33800)
                             // bit 1 = device one  (speedo display)
                             // bit 2 = device 2

unsigned char pad301;	// pad byte 

} inputs3;

#pragma ROM_VAR INP_ROM

// flash copy of inputs - initialized

#define SECTOR_BYTES         1024 // bytes
#define SECTOR_WORDS         ((int)(SECTOR_BYTES/2))
#define N_SECTORS(theStruct) ((int)((sizeof(theStruct)+SECTOR_BYTES-1)/SECTOR_BYTES))
#define N_PADDING(theStruct) ((int)(N_SECTORS(theStruct)*SECTOR_BYTES - sizeof(theStruct)))


// Assign values to the in1flash parameters
const inputs1 in1flash = {
1,                // input capture edge for VSS, 0=rising edge, 1 = falling edge
{12,              // vss_table1[MPH no = 0] , MPH (KPH), use for 16x9 auto shift table
  18,26,36,48,54,60,66,72,80,90,100,110,120,130,140},
{20,              // LOAD_table1[LOAD/tps no = 0],  kPa, for use in 16x9 auto shift table
  30,40,50,60,70,80,90,100},
{-400,            // temp_table[TEMP no = 0],  deg x 10
    -200,0,200,400,600,800,1000,1300,1600},
{2290,3060,1630,1000,700,600,500,450,400}, // gear_table[], gear ratios * 1000	 (0th is reverse)
1,                // CAN_enabled = 1 = yes; 
0,                // menuDisplay, 0=show all menus
700,              // noTCC_temp (degrees x10)
900,              // noTCC_load (kpa x10)
20,               // LUF taper time to go to 100% PWM DC (sec x10)
25,               // PulseTol,     % tolerance for next input pulse timing during    
                  // normal running    
3080,2600,        // axle_ratio, tire_diam
100,              // min_dcl_spd
                  // (switch to auto in in manual and not first gear)
0,                // trans_type, 0=4L60E
1,                // shift_mode; 0=manual, 1=auto (sequential), 2=auto (skip shift)
2,                // LPshiftOptn; 0=table, 1=OFF, 2=shift limit
1,                // error checking enabled by default
6500,             // spare12
30,               // PWM period4 (usec) for speedo output
                  // less than 0.1*debounce or greater than 0.9*debounce)
40,               // num_teeth, divide factor for input VSS pulses to revs, 40 teeth for 4L60E VSS
10417,            // TCC_PWM_Pd (us) (= 96 Hertz)
3413,             // PC_PWM_Pd,  Injector PWM period (us) (=293 Hz)
20000,            // SOL32_PWM_Pd, Solenoid C period (us) (=50 Hz)
0,                // BatFac,  Battery PWM correction factor (% (x10) per volt)
0,7,              // ms2canID, ms2varBLK (outpc. on ms2) 
20,               // PCneutral
115200,           // baud rate
3,                // board type
1,                // mycan_id of this board, always 0 for ECU, 1 for all others until configured
60,               // can_var_rate; rate at which to get CAN variables from MS-II -
                  // 8 bytes (4 words) per can_var_rate in 0.128 ms tics;
                  // can get all 112 bytes of outpc in 14 x canvarate tics.

/* 16x9 Target Gear Table 1 for Auto Mode **/   
{{1,            // auto_table1[LOAD no=0][MPH no=0],  target gear
    2,2,3,3,3,3,4,4,4,4,4,4,4,4,4},
{1,             // auto_table1[LOAD no=1][MPH no=0],  target gear
    2,2,3,3,3,3,4,4,4,4,4,4,4,4,4},
{1,             // auto_table1[LOAD no=2][MPH no=0],  target gear
    1,2,2,3,3,3,4,4,4,4,4,4,4,4,4},
{1,             // auto_table1[LOAD no=3][MPH no=0],  target gear
    1,2,2,3,3,3,3,3,4,4,4,4,4,4,4},
{1,             // auto_table1[LOAD no=4][MPH no=0],  target gear
    1,1,2,2,3,3,3,3,4,4,4,4,4,4,4},
{1,             // auto_table1[LOAD no=5][MPH no=0],  target gear
    1,1,2,2,3,3,3,3,4,4,4,4,4,4,4},
{1,             // auto_table1[LOAD no=6][MPH no=0],  target gear
    1,1,2,2,2,3,3,3,3,3,4,4,4,4,4},
{1,             // auto_table1[LOAD no=7][MPH no=0],  target gear
    1,1,2,2,2,2,3,3,3,3,4,4,4,4,4},
{1,             // auto_table1[LOAD no=8][MPH no=0],  target gear
    1,1,2,2,2,2,3,3,3,3,3,4,4,4,4}},

/* 16x9 Pressure Control PWM Table **/   
{{40,             // pc_table[LOAD no=0][MPH no=0], PWM%
    40,40,40,40,40,40,40,40,40,40,40,40,40,40,40},
{37,             // pc_table[LOAD no=1][MPH no=0],  PWM%
    37,37,37,37,37,37,37,37,37,37,37,37,37,37,37},
{34,             // pc_table[LOAD no=2][MPH no=0],  PWM%
    34,34,34,34,34,34,34,34,34,34,34,34,34,34,34},
{30,             // pc_table[LOAD no=3][MPH no=0],  PWM%
    30,30,30,30,30,30,30,30,30,30,30,30,30,30,30},
{26,             // pc_table[LOAD no=4][MPH no=0],  PWM%
    26,26,26,26,26,26,26,26,26,26,26,26,26,26,26},
{23,             // pc_table[LOAD no=5][MPH no=0],  PWM%
    23,23,23,23,23,23,23,23,22,21,20,20,20,20,20},
{20,             // pc_table[LOAD no=6][MPH no=0],  PWM%
    20,20,20,20,20,20,20,20,20,19,18,18,18,18,18},
{18,             // pc_table[LOAD no=7][MPH no=0],  PWM%
    18,18,18,18,18,18,17,16,15,14,13,13,13,13,13},
{13,             // pc_table[LOAD no=8][MPH no=0],  PWM%
    13,13,13,13,13,13,12,11,10,9,8,8,8,8,8}},
    
2100,        // trans temperature limit
2002,        // pulse/mile for speedometer output
0,           // 0 = no rpm_check, 1 = check rpm
35,          // gear speed hysteresis (mph x10)
0,           // TCC follow mode for sp1 (0 is don't engage with TCC in any gear)
20,          // max shift pressure (PC DC%)
10,          // pressure delay ( 10 = ~85 milliseconds)
0,           // TCC follow mode for sp2 (0 is don't engage with TCC in any gear)
9,			 // load_adj--> 1 = 00001001
			 // 			bit 0: brake load adjust 0=usual rate, 1=slow rate
			 //			 	bit 1: decel load adjust 0=usual load, 1=limit load
			 // 			bit 2: rising load adjust 0=usual rate, 1=rise quickly
			 //             bit 3: tps at WOT: 0=usual rate, 1=rise quickly 
1000,        // LOADshortcount (400 = ~1/10 sec)
60,          // LOADlongcount  (60 = ~8 sec)
2000,        // LOADmult
0,           // LOADzero
410,         // minTCCspeed

{0,          // PCtemp[],  PWM% adjustment for temperature
     0,0,0,0,0,0,0,0,0},

//------------------------- 4 3 2 1 R N P 
79,          // Output1 = 0 1 0 0 1 1 1 1 = 79 (solA for the 4L60E) 
31,          // Output2 = 0 0 0 1 1 1 1 1 = 31 (solB for the 4L60E) 
96,          // Output3 = 0 1 1 0 0 0 0 0 = 96 (3-2 soln for the 4L60E)
0,           // Output4 = 0 0 0 0 0 0 0 0
 
90,          // Output3 (SolC duty cycle when on)

400,600,	// spare port 1,2 speed thresholds
40,60, 		// spare port 1,2 speed hysteresis
1,1,    	// spare port 1,2 speed condition

5000,5000,	// spare port 1,2 rpm thresholds
200,200, 	// spare port 1,2 rpm hysteresis
0,0,    	// spare port 1,2 rpm condition

800,800,	// spare port 1,2 load thresholds
100,100,	// spare port 1,2 load hysteresis
0,0,    	// spare port 1,2 load condition

3,3,		// spare port 1,2 gear thresholds
0,0,    	// spare port 1,2 speed condition

//------------------------ 4 3 2 1 R N P 
115,         // Input1 =   1 1 1 0 0 1 1 (pressure switch manifold switch A)
56,          // Input2 =   0 1 1 1 0 0 0 (pressure switch manifold switch B)
31,          // Input3 =   0 0 1 1 1 1 1 (pressure switch manifold switch C)
0,           // Input4 =   0 0 0 0 0 0 0

1,           // forceAUTOoptn (1 = always)
819,         // ADC count in park for Ford  - 4.0V 
717,         // ADC count in neutral for Ford  - 3.5V
614,         // ADC count in reverse for Ford  - 3.0V
205,         // ADC count in 1 for Ford  - 1.0V
307,         // ADC count in 2 for Ford  - 1.5V
410,         // ADC count in 3 for Ford  - 2.0V
512,         // ADC count in drive for Ford (4th)  - 2.5V
980,         // if mlever_mode=1, ADC count in 5th
990,         // if mlever_mode=1, ADC count in 6th
1000,        // if mlever_mode=1, ADC count in 7th
1010,        // if mlever_mode=1, ADC count in 8th

0,           // swBDC_OD: 0 use gear lever for 4th, 1 = use separate switch

0,           // SlipConditions: 	
			 // 0 = No slip adjustments
			 // 1 = Adjust spark only
			 // 2 = Adjust spark AND line pressure 
			 // 3 = Adjust spark THEN line pressure
			 // 4 = Adjust line pressure only
			 // 5 = Adjust line pressure then spark
0,           // clutch1 upshift pattern
0,           // clutch2 upshift pattern
0,           // clutch1 downshift pattern
0,           // clutch2 downshift pattern

1000,        // FWD_factor (x1000)

-50,-50,0,-10,-30,// timing retards - up/downshift, 2nd, 3rd, 4th
900,              // timing retard above load (kpax10)
0,500,1000,30,95, // LUF off state, loLoad, hiLoad, loPWM%, hiPWM%

204,         // ADC to digital on/off threshold = 1 Volt
1250,        // shifter accumulator - button sensitivity (lower is more sensitive)
50,          // Hysteresis enable minimum speed (5 mph/kph)
80,          // VSS interrupt mask (0 is disabled)
7,           // binary 00000111 - use FWD to switch speedo, use temp sensor, use as temperature not ADC
100,         // LUF_on percentage
30,          // minimum speed (x10)
100,         // maximum VSS errors before a reset is invoked
0,           // line_units - 0=use table, 1=use ADC, 2=use as engine brake input, 3=use 2 point calcs
2400,        // injector flow (pphx10) of ALL injectors, open_time (msecx100)
100,	     // open time (x100)
6073,        // fuel density (pounds/gallon)
1,           // number of squirts (divide by 2 if alternating)        
500,         // PWM refresh interval (msec)
0,           // PWM refresh duration (msec)
10000,       // PC dither interval (msec)
0,           // PC dither duration (msec)
0,           // output1/5/6 off duration (in 0.128 clock tics)
100,         // output1/5/6 on duration (in 0.128 clock tics)
1,           // Brake ON polarity (0=low, 1=high)
952,         // 952/1024 = 93% TPS for WOT
0,           // ISS mask (%) - 0 is disabled
0,           // ic2 usage (0 = not used); ic2 edge = 0 - rising 
31,          // ISS/tach divider (teeth/rev)
0,           // activate_xrate: 8 bits, one for each CAN outmsg.  When bit m set to 1, 
             // message m will be auto xmitted at rate outmsg[m].xrate.
4,           // number of forward gears
-35,         // fifth gear retard (degx10)
-40,         // sixth gear retard (degx10)
-45,         // seventh gear retard (degx10)
-50,         // eighth gear retard (degx10)
5,           // Fuel/Spark/Idle adjustment block on MS-II
946,         // Fuel/Spark/Idle adjustment variable offset for MS-II
15,          // LED1 bit pattern - 00001111 
30,          // LED2 bit pattern - 00011110
60,          // LED3 bit pattern - 00111100
120,         // LED4 bit pattern - 01111000
0,           // LEDN (neutral) bits 0-3 are for LED1 to LED4 - 00000000
165,         // LEDR (reverse) bits 0-3 are for LED1 to LED4 , bits 4-7 are alternating pattern - 1010 0101
10,          // IACAdjngear - IAC adjustment when gear==1 and VSS==0 and TPS==0
50,          // AdvAdjngear - Ignition Advance adjustment when gear==1 and VSS==0 and TPS==0
50,          // idle tps adc (less than this)
0,           // can_testout; 0 = entire outpc, 1 = use outmsg
60,          // min_manual_speed
0,           // shift button polarity (0=active low, 1=active high)
0,           // TCC_PWM
0,           // spareInputs
0,           // Output5
0,           // Output6
0,           // PWM_enable (for bit banged shift outputs)
1500,        // max speed x10 (for VSS filtering)
15,          // resetTime (1.5 seconds)
75,          // TCC hysteresis time (7.5 seconds)
5000,        // TCC full time apply rpm
4,           // TCC full time apply gear
4500,        // TCC full time unlock
4,           // VSSsmoothFactor
0,           // TCC_highGear: 0 = use normal limits, 1 = always appled in high gear
0,           // TCCupshift - same bit patterns as Outputs - 1 is unlock, 0 is lock
             // to 2nd = 0x10, 3rd = 0x20, 4th = 0x40, 5th = 0x80, 
             // 6th = 0x100, 7th = 0x200, 8th = 0x400
             // 8 7 6 5 4 3 2 1 R N P 
             // 0 0 0 0 1 0 0 1 1 1 1

{30,         // maxUPpressure[8]   Maximum pressure when shifting up to the specified gear
30,30,30,30,30,30,30},

{30,         // maxDWNpressure[8]  Maximum pressure when shifting down to the specified gear
30,30,30,30,30,30,30},

{0,          // solCupshiftPWM[7] - upshift PWM% for output3/solC/(PWMDTY1/PWMPER1*100%)
             // bit    0x01 0x02 0x04 0x08 0x10 0x20
             // shift  1-2  2-3  3-4  4-5  6-7  7-8
45,0,0,0,0,0},

{0,          // solCdwnshiftPWM[7] - downshift PWM% for output3/solC/(PWMDTY1/PWMPER1*100%)
             // bit    0x01 0x02 0x04 0x08 0x10 0x20
				     // shift  2-1  3-2  4-3  5-4  7-6  8-7
45,0,0,0,0,0},
2000,         // digital switch debounce
0,            // WOT_dur hysteresis blocking duration in seconds
0,            // manual level mode:
              // 0 for swA,swB, swC  (GM)
              // 1 for voltage based (Ford) 
0,            // shift button mode 0 = two grounded input, 1 = single ADC                               
971,          // Volts for neither button pressed (ADC count)
506,          // Volts for sw1 button pressed  (ADC count)
639,          // Volts for sw2 button pressed  (ADC count)
315,          // Volts for sw1 + sw2 button pressed  (ADC count)
53,           // Volts for both buttons pressed
0,            // 0 = GPIO, 1= microTCU
0,            // SP1 mode - 0 = spare port, 1 = PWM, 2 = TCC, 3 = shift solenoid
0,            // SP2 mode - 0 = spare port, 1 = PWM, 2 = TCC, 3 = shift solenoid
{0,           // PWM lookup table for SP1 when SP1_mode == 1
10,20,30,40,50,60,70,80,90,95,100}, 
{0,           // PWM lookup table for SP2 when SP2_mode == 1
10,20,30,40,50,60,70,80,90,95,100}, 
{300,         // PWM lookup table index for SP1 when SP1_mode == 1
400,500,600,700,800,900,1000,1100,1200,1300,1400}, 
{300,         // PWM lookup table index for SP2 when SP2_mode == 1
400,500,600,700,800,900,1000,1100,1200,1300,1400}, 
0,            // sp1_index_mode - value to compare to sp1_pwm_index - 0 = load, 1 = speed, 2 = rpm, 3 = clt, 4 current gear
0,            // sp2_index_mode - value to compare to sp2_pwm_index - 0 = load, 1 = speed, 2 = rpm, 3 = clt, 4 current gear
2,            // sp1_active - if SP1 is PWM, 0 - always on, 1 - on except shifting, 2 on only during shift
2,            // sp2_active - if SP2 is PWM, 0 - always on, 1 - on except shifting, 2 on only during shift
0,            // sp1_off - if SP1 is PWM, this is the 'off state' PWM%
0,            // sp2_off - if SP2 is PWM, this is the 'off state' PWM%
78,           // sp1_pwm_per - the PWM period for spare port 1 in 0.000128 second ticks (78 = 100 Hz)
78,           // sp2_pwm_per - the PWM period for spare port 2 in 0.000128 second ticks (78 = 100 Hz) 
31,           // Output7 = 0 0 0 1 1 1 1 1 = 31 (solB for the 4L60E)  
0,            // output2 off duration (in 0.128 clock tics)
100,          // output2 on duration (in 0.128 clock tics)   
3,            // minTCC_gear 
100,          // TCC follow duration for sp1 (100 is 1.00 second)
{20,            // LOAD_table2[LOAD/tps no = 0],  kPa, for use in 16x9 auto shift table 2
  30,40,50,60,70,80,90,100},
{12,            // vss_table2[MPH no = 0] , MPH (KPH), use for 16x9 auto shift table 2
  18,26,36,48,54,60,66,72,80,90,100,110,120,130,140},

/* 16x9 Target Gear Table 2 for Auto Mode **/   
{{1,            // auto_table2[LOAD no=0][MPH no=0],  target gear
    2,2,3,3,3,3,4,4,4,4,4,4,4,4,4},
{1,             // auto_table2[LOAD no=1][MPH no=0],  target gear
    2,2,3,3,3,3,4,4,4,4,4,4,4,4,4},
{1,             // auto_table2[LOAD no=2][MPH no=0],  target gear
    1,2,2,3,3,3,4,4,4,4,4,4,4,4,4},
{1,             // auto_table2[LOAD no=3][MPH no=0],  target gear
    1,2,2,3,3,3,3,3,4,4,4,4,4,4,4},
{1,             // auto_table2[LOAD no=4][MPH no=0],  target gear
    1,1,2,2,3,3,3,3,4,4,4,4,4,4,4},
{1,             // auto_table2[LOAD no=5][MPH no=0],  target gear
    1,1,2,2,3,3,3,3,4,4,4,4,4,4,4},
{1,             // auto_table2[LOAD no=6][MPH no=0],  target gear
    1,1,2,2,2,3,3,3,3,3,4,4,4,4,4},
{1,             // auto_table2[LOAD no=7][MPH no=0],  target gear
    1,1,2,2,2,2,3,3,3,3,4,4,4,4,4},
{1,             // auto_table2[LOAD no=8][MPH no=0],  target gear
    1,1,2,2,2,2,3,3,3,3,3,4,4,4,4}},
{20,            // LOAD_PCtable[LOAD/tps no = 0],  kPa, for use in 16x9 auto shift table 2
  30,40,50,60,70,80,90,100},
0,			    // LEDP - LED pattern in Park
102,            // Volt1 in ADC counts (102 = 0.5 Volts)
922,            // Volt2 in ADC counts (922 = 4.5 Volts)
0,              // Press1 (psi x 10)
2900,           // Press2 (psi x 10)
5,              // mleverdead - plus/minus dead band for manual lever voltage
1000,           // Sretard_load (x10) - Minimum load to apply timing retard based on internal slip
0,              // Slip_adv - advance to be subtracted from advance when internal slip
0,              // Slip_fuel - fuel to add when slipping
110,            // Slip_threshold - minimum slip to apply adjustments to cut power
512,            // INdbug value for testing as ADC count 
				// == 2.504 Volts if volts
// 1000 bytes to here = 125 * 8;
0,            	// PCreverse = 100% line pressure
0,				// PCrvrseOptn = 0 = Use fixed value
0,				// leverErr: 0 is ignore all errors, 1 is check all errors, 
0,              // minADCLeverVolts	 (ADC count)
1023,			// maxADCLeverVolts	 (ADC count)
20,             // time between slip line and slip spark adjustments (10ths) or vice-versa
0, 				// NAGmode   0 = regular mode (current gear from solenoid states); 
				//			 1 = NAG mode (current gear from last shift)
0,				// burnout_mode 0 - use 1st and 2nd, 1- use 2nd only
2,				// burnout_maxgear - highest gear for burn-out mode
20,				// brake load
120,            // decel mode TPS threshold
0,            	// VSS location 0 - output shaft, 1 - wheel
100,            // TCC follow duration for sp2 (100 is 1.00 second)
};				// pad as required to make total page size a multiple of 8 for TS CAN

// Pad in1flash to end of sector
const unsigned char in1padding[N_PADDING(inputs1)] ={0}; 

// Put values in in2flash spare variables
const inputs2 in2flash = { 
{12,              // vss_PCtable[MPH no = 0] , MPH (KPH), use for 16x9 auto shift table 2
  18,26,36,48,54,60,66,72,80,90,100,110,120,130,140},
{0,              // speedoCorr[MPH no = 0] (%x10) use for 16 element speedo correction table
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},  
{12,              // speedoCorrBin[MPH no = 0] , MPH (KPH), use for 16 element speedo correction table
  18,26,36,48,54,60,66,72,80,90,100,110,120,130,140},

0,   // pad201
0,   // pad202
0,   // pad203
0,   // pad204
0,   // pad205
0,   // pad206
0,   // pad207
0,   // pad208
0,   // pad209
0,   // pad210
0,   // pad211
0,   // pad212
0,   // pad213
0,   // pad214

/*
0,   // SPIsoln1 (0 = all off)  
32,  // SPI0BR for device 0  = 0x20 = 4Mhz
80,  // SPI0CR1 for device 0 = 0x50
2,   // SPI0CR2 for device 0 = 0x02
32,  // SPI0BR for device 1
80,  // SPI0CR1 for device 1
2,   // SPI0CR2 for device 1
32,  // SPI0BR for device 2
80,  // SPI0CR1 for device 2
2,   // SPI0CR2 for device 2
32,  // SPI0BR for device 3
80,  // SPI0CR1 for device 3
2,   // SPI0CR2 for device 3
0,	 // spi_enable - no devices enabled by default
*/

0,	 // sp0_mode: 0 - speedo, 1 - PWM, 2 - TCC, 3 - shift solenoid
0, 	 // sp0_active: 0 - always on, 1 - on except shifting, 2 - on only during shift
{12, // sp0_16Bins[MPH no = 0] , use for 16x9 auto shift table 2
  18,26,36,48,54,60,66,72,80,90,100,110,120,130,140},
{20, // sp0_9Bins[MPH no = 0] , use for 16x9 auto shift table 2
  30,40,50,60,70,80,90,100},
/* 16x9 Spare Port 0 PWM Table **/   
{{99,             // pc_table[LOAD no=0][MPH no=0], PWM%
    99,99,99,99,99,99,99,99,99,99,99,99,99,99,99},
{97,             // sp0_table[LOAD no=1][MPH no=0],  PWM%
    97,97,97,97,97,97,97,97,97,97,97,97,97,97,97},
{94,             // sp0_table[LOAD no=2][MPH no=0],  PWM%
    94,94,94,94,94,94,94,94,94,94,94,94,94,94,94},
{90,             // sp0_table[LOAD no=3][MPH no=0],  PWM%
    90,90,90,90,90,90,90,90,90,90,90,90,90,90,90},
{86,             // sp0_table[LOAD no=4][MPH no=0],  PWM%
    86,86,86,86,86,86,86,86,86,86,86,86,86,86,86},
{73,             // sp0_table[LOAD no=5][MPH no=0],  PWM%
    73,73,73,73,73,73,73,73,72,71,70,70,70,70,70},
{60,             // sp0_table[LOAD no=6][MPH no=0],  PWM%
    60,60,60,60,60,60,60,60,60,59,58,58,58,58,58},
{48,             // sp0_table[LOAD no=7][MPH no=0],  PWM%
    48,48,48,48,48,48,47,46,45,44,43,43,43,43,43},
{39,             // sp0_table[LOAD no=8][MPH no=0],  PWM%
    39,38,37,36,35,35,34,34,34,33,33,33,33,32,32}},
10417, 	// sp0_per
0,      // sp0_off
0,  	// sp0_9index_mode	 (0=load)
1,		// sp0_16_index_mode (1=speed)
0, 		// SPIsoln2
0,		// clutch0up
0,		// clutch0dwn
43690,  // Output8 pattern  = 1010 1010 1010 1010
0,      // LED flash - bit0 = 0 means no flashing
1,      // enable nested interrupts (0=no)
0,      // SlipOptn - 0=TCC slip
  
{{10,    // TPStable - table to convert TPS and RPM to load
  20,20,30,40,50},  
{10,   // TPStable - table to convert TPS and RPM to load
  16,23,32,45,60}, 
{10,  // TPStable - table to convert TPS and RPM to load
  12,25,35,55,70}, 
{30,  // TPStable - table to convert TPS and RPM to load
  45,50,55,70,80}, 
{40,  // TPStable - table to convert TPS and RPM to load
  70,75,80,85,90}, 
{55,  // TPStable - table to convert TPS and RPM to load
  80,90,100,100,100}},
  
{500,   // TPStable RPM bins
  800,1000,1200,3000,6000}, 
{0,     // TPStable TPS bins
  120,250,500,750,1000},  
25,		// TPSmap - use 75% TPS / 25% MAP
{20,     // Shift completion delay bins
  30,50,70,80,90,95,100},
{100,     // Shift completion delay values
  65,45,32,25,20,20,20},
   
0,	 // sp3_mode: 0 - line pressure, 1 - PWM, 2 - TCC, 3 - shift solenoid
1, 	 // sp3_active: 0 - always ON, 1 - ON if RPM > 300, else OFF
{10, // sp3_16Bins[load no = 0] , use for 16x9 spare port 3
  18,26,36,48,54,60,66,72,78,84,90,93,96,98,100},
{-1, // sp3_9Bins[gear no = 0] , use for 16x9 spare port 3
  0,1,2,3,4,5,6,7},
/* 16x9 Spare Port 3 PWM Table */   
{{60,             // pc_table[gear no=0][load no=0], PWM% (not 100-PWM%)
    50,40,30,20,10,8,6,4,3,2,1,0,0,0,0},
{60,             // sp3_table[gear no=1][load no=0], PWM%
    50,40,30,20,10,8,6,4,3,2,1,0,0,0,0},
{60,             // sp3_table[gear no=2][load no=0], PWM%
    50,40,30,20,10,8,6,4,3,2,1,0,0,0,0},
{60,             // sp3_table[gear no=3][load no=0], PWM%
    50,40,30,20,10,8,6,4,3,2,1,0,0,0,0},
{60,             // sp3_table[gear no=4][load no=0], PWM%
    50,40,30,20,10,8,6,4,3,2,1,0,0,0,0},
{60,             // sp3_table[gear no=5][load no=0], PWM%
    50,40,30,20,10,8,6,4,3,2,1,0,0,0,0},
{60,             // sp3_table[gear no=6][load no=0], PWM%
    50,40,30,20,10,8,6,4,3,2,1,0,0,0,0},
{60,             // sp3_table[gear no=7][load no=0], PWM%
    50,40,30,20,10,8,6,4,3,2,1,0,0,0,0},
{60,             // sp3_table[gear no=8][load no=0], PWM%
    50,40,30,20,10,8,6,4,3,2,1,0,0,0,0}},
3413, 	// sp3_per
0,      // sp3_off
4,  	// sp3_9index_mode	 (4=gear)
0,		// sp3_16_index_mode (0=load)
43690,  // Output9 pattern  = 1010 1010 1010 1010
0, 		// clutch3up
0,		// clutch3dwn
{0,
	0,0,0,0,0,0,0}, // Slip_limit[8]
25,  	// decel_Delay (in tenths of a second)
19,		// lever_timer (19 + 1 from INI = 20 == 2.0 seconds)
1,   	// maxDeltaLever maximum lever position change in forward gears
0,		// canVSSoffset - 0 to disable
112,	// msOutpcBytes
50, 	// TCC_hyst
50,		// speedoutDC
{ 80,		// gear_indic_freq[11]
	120,160,200,240,360,680,1320,4000,4400,4800 },
{ 10,		// gear_indic_dc[11]
	20,30,40,50,60,70,75,80,90,100 },
0,		// gear indicator mode 0=current gear, 1=manual gear

{ 1000,	   // ISS ratios[9] (x1000)
	  1000,1000,1000,1000,1000,1000,1000,1000 },

2,		// burn-out mode maximum gear
10,		// NAGrpmTol
72,		// NAGout1 digital bit pattern
32,		// NAGout2 digital bit pattern
16,		// NAGout3 digital bit pattern
2000,	// NAGwait
16,		// NAGretard
0,		// unused
600,	// decel Load ceiling (x10)
1000,	// inter_ratio (x1000)
8,		// gear tolerance
{ 300,  // 8 downshift load bins
	400, 500, 600, 700, 800, 900, 1000 }, 	
{ 200,  // 8 upshift load bins
	300, 400, 500, 600, 700, 800, 1000 },
		
// 14 x 8 RPM Shift Table
// ---------------------
// rows 0 - 6  are the upshift   values for shifts 1/2 ... 7/8 (i.e. current_gear - 1)
// rows 7 - 13 are the downshift values for shifts 2/1 ... 8/7 (i.e. current_gear + 5)
 // RPM based shift table
// Upshifts
{
	
 2500, // upshift 1/2  -- row 0
	 2900, 3300, 3800, 4100, 4500, 5000, 6000,
 2400, // upshift 2/3  -- row 1
	 2800, 3200, 3700, 4000, 4500, 5000, 6000,
 2300, // upshift 3/4  -- row 2
	 2700, 3100, 3600, 4000, 4500, 5000, 6000,
 2200, // upshift 4/5  -- row 3
	 2600, 3000, 3600, 4000, 4000, 5000, 6000,
 2100, // upshift 5/6  -- row 4
	 2500, 2900, 3600, 4000, 4500, 5000, 6000,
 2000, // upshift 6/7  -- row 5
	 2400, 2800, 3600, 4000, 4500, 5000, 6000,
 2000, // upshift 7/8  -- row 6
	 2400, 2800, 3600, 4000, 4500, 5000, 6000,
// Downshifts	 
 1400, // downshift 2/1 -- row 7
	 1650, 1900, 2000, 2200, 2400, 2600, 2800,
 1450, // downshift 3/2 -- row 8
	 1700, 1900, 2000, 2200, 2400, 2600, 2800,
 1500, // downshift 4/3 -- row 9
	 1750, 1900, 2000, 2200, 2500, 2700, 2900,
 1500, // downshift 5/4 -- row 10
	 1750, 1900, 2000, 2200, 2400, 2800, 3000,
 1500, // downshift 6/5 -- row 11
	 1750, 1900, 2000, 2200, 2400, 2750, 3100,
 1500, // downshift 7/6 -- row 12
	 1750, 1900, 2000, 2200, 2500, 2900, 3200,
 1500, // downshift 8/7 -- row 13
	 1750, 1900, 2000, 2300, 2600, 2900, 3200  
},
1000,   // CANbus VSS adjustment

2,      // CANbus offset for outpc PW1 (2)
6,      // CANbus offset for outpc RPM (6)
18,     // CANbus offset for outpc MAP (18)
24,     // CANbus offset for outpc TPS (24)
26,     // CANbus offset for outpc vBatt (26)

0,      // CANbus offset for outmsg PW1 (0)
2,      // CANbus offset for outmsg RPM (2)
4,      // CANbus offset for outmsg MAP (4)
6,      // CANbus offset for outmsg TPS (6)
8,      // CANbus offset for outmsg vBatt (8)
0,      // CANbus offset for outmsg speedo (-)
0,      // VSS_ISR 0 = old, 1 = new
0,      // CANbus VSS offset
35,     // hyst_delay 3.5 seconds
{ 1500, 1500, 1500, 1500, 1500, 1500, 1500	  // under rev limit by gear
},
{ 6000, 6000, 6000, 6000, 6000, 6000, 6000	  // RPM limit by gear
},
0x00,	// test_mode
		// bit 0    (0x01) - 0 = disable testing, 1 = enable testing
		// bit 1    (0x02) - 1 = test Input 1 - use IN1dbug as ADC input for PAD00
		// bit 2    (0x04) - 1 = test Input 2 - use IN2dbug as ADC input for PAD01
		// bit 3    (0x08) - 1 = test Input 3 - use IN3dbug as ADC input for PAD03
		// bit 4    (0x10) - 1 = test temperature - use TMPdbug as ADC input for temperature (PAD02)
		// bit 5    (0x20) - 1 = test VSS - use VSSdbug as speed period for VSS
		// bit 6    (0x40) - 1 = test ISS - use ISSdbug as period for ISS
		// bit 7    (0x80) - 1 = test Paddle UP - use TSTdbug {bit 0} as input for upshift paddle (PT6)
		// bit 8   (0x100) - 1 = test Paddle DOWN - use PDNdbug as ADC input for downshift paddle (PAD06)
		// bit 9   (0x200) - 1 = test Brake - use BRKdbug as input for brake (PAD07)
		// bit 10  (0x400) - 1 = test line pressure - use LPdbug as ADC input for temperature (PAD04)
		// bit 11  (0x800) - 1 = test 4WD - use TSTdbug {bit 1} as 4WD input (PE1)
		// bit 12 (0x1000) - 1 = test non-CANbus Load - use nCANdbug as ADC input

255,    // IN1dbug  -  Input 1 ADC count (/4)
0,      // IN2dbug  -  Input 2 ADC count (/4)
0,		// IN3dbug  -  Input 3 ADC count (/4)
40, 	// TMPdbug  -  Temperature ADC count (/4)
60,		// VSSdbug  -  mph
50,		// ISSdbug  -  ISS
255,	// PDNdbug  -  Upshift paddle ADC count (/4)
0x03,	// TSTdbug  -  bit 0 (0x01) - Paddle Up state
		//             bit 1 (0x02) - 4WD/Table shift state
0,		// BRKdbug  -  Brake ADC count (/4)
100,    // LPdbug   -  Line Pressure ADC count (/4)
100,	// nCANdbug -  non-CAN Load ADC count (/4)

1000,	// CANloaddbug
2500,	// CANrpmdbug
131,	// CANbattdbug
};

// Pad in2flash to end of sector     
const unsigned char in2padding[N_PADDING(inputs2)] ={0};
                                // 1024 bytes = 1 block flash

const inputs3 in3flash = {
155,  		// pad316
156,  		// pad315
157,  		// pad314
158,  		// pad313
159,  		// pad312
160,  		// pad311
161,  		// pad310
162,  		// pad309
163,  		// pad308
164,  		// pad307
165,  		// pad306
166,  		// pad305
167,  		// pad304
168,  		// pad303
169,  		// pad302
170,  		// pad301

0, 			// SPIsoln1;  
			// bit 0 = sol A = Output 1 (1=use SPI)
            // bit 1 = sol B = Output 2 (1=use SPI)
            // bit 2 = sol C = Output 2 (1=use SPI)
            // bit 3 = sol D = Output 2 (1=use SPI)
            // bit 4 = sol E = Output 2 (1=use SPI)
            // bit 5 = sol F = Output 2 (1=use SPI)
            // bit 6 = TCC (1=use SPI)
            // bit 7 = PC  (1=use SPI)

52, 		// D0BR;    // SPI0BR baud rate for device 0
  			// set up SPI for MC33800
  			// SPICR1
  			// bit 0: LSBFE		= 0
  			// bit 1: SSOE		= 1  (turn on slave select output enable (requires MODFEN == 1))
  			// bit 2: CPHA		= 0
  			// bit 3: CPOL		= 0
  			// bit 4: MSTR		= 1  (Master mode)
  			// bit 5: SPTIE		= 0
  			// bit 6: SPE	    = 1  (SPI enabled, port pins are dedicated to SPI functions.)
  			// bit 7: SPIE     	= 0
	  
0x12,		// SPICR2
  			// bit 0: SPC0   	= 0
  			// bit 1: SPISWAI	= 1 (SPI clock operates normally in wait mode)
  			// bit 2: 0		 	= 0
  			// bit 3: BIDIROE	= 0
  			// bit 4: MODFEN	= 1 (Use SS pin)
  			// bit 5: 0		 	= 0
  			// bit 6: 0		 	= 0
  			// bit 7: 0		 	= 0
  			
0, 			// D0CR1;   // SPI0CR1 for device 0
0,			// D0CR2;   // SPI0CR2 for device 0
0,			// D1BR;    // SPI0BR baud rate for device 1
0,			// D1CR1;   // SPI0CR1 for device 1
0,			// D1CR2;   // SPI0CR2 for device 1
0,			// D2BR;    // SPI0BR baud rate for device 2
0,			// D2CR1;   // SPI0CR1 for device 2
0,			// D2CR2;   // SPI0CR2 for device 2
0,			// D3BR;    // SPI0BR baud rate for device 3
0,			// D3CR1;   // SPI0CR1 for device 3
0,			// D3CR2;   // SPI0CR2 for device 3
0,			// spi_enable  
			// spi device enable
            // 0 = disabled, 1 = enabled 
            // bit 0 = device zero (MC33800)
            // bit 1 = device one  (speedo display)
            // bit 2 = device 2
            // bit 3 = device 3
            
170, 		// pad byte
};

// Pad in3flash to end of sector     
const unsigned char in3padding[N_PADDING(inputs3)] ={0};
                                // 1024 bytes = 1 block flash