// This header defines all the I/O port assignments so that
// the main.c, etc. are clear of direct hardware references.

/** I/O Port Quick_Start ******************************************
    --------------------

Function             Port    I/O    ON    OFF   READ               NOTES
--------             ----    ---    --    ---   ----               -----

Brake Sense			 PAD07	  I     --    --    (PORTAD0 & 0x80)   - on/off			
Paddle DOWN			 PAD06    I     --    --    (PORTAD0 & 0x40)   - on/off   	
non-CAN MAP/TPS/MAF  PAD05    I     --    --    ATD0DR5            - ADC count   
line pressure sensor PAD04    I     --    --    ATD0DR4            - ADC count     
Input3				 PAD03	  I     --    --    ATD0DR3            - ADC count  
Temp Sensor			 PAD02	  I     --    --    ATD0DR2            - ADC count    
Input2				 PAD01	  I     --    --    ATD0DR1            - ADC count  
Input1				 PAD00	  I     --    --    ATD0DR0            - ADC count  

Spare Output1        PT7      O     spr1(1);  *pPTTpin[7] &= ~0x80; ---
Paddle UP			 PT6      I     --    --    PTT & 0x40         - on/off
ISS                  PT5      I     Interrupt ISS_timer()          - interrupt called on edge     			
Speedo Output		 PT4	  O		PWMDTY4 = 0;          PWMDTY4 = PWMPER4;    PWMDTY4
TCC					 PT3      O		PWMDTY3 = 0;          PWMDTY3 = PWMPER3;    PWMDTY3
PC					 PT2	  O	    PWMDTY2 = 0;          PWMDTY2 = PWMPER2;    PWMDTY2
Output3				 PT1      O     PWMDTY1 = 0;          PWMDTY1 = PWMPER1;    PWMDTY1
VSS					 PT0	  I     Interrupt VSS_timer()          - interrupt called on edge

LED4  				 PB4      O     PORTB |= 0x10;        PORTB &= ~0x10;       ---

LED3  	  			 PM5      O     *pPTMpin[5] |= 0x20;  *pPTMpin[5] &= ~0x20; ---	   
LED1				 PM4      O     *pPTMpin[4] |= 0x10;  *pPTMpin[4] &= ~0x10; ---
LED2 				 PM3      O     *pPTMpin[3] |= 0x08;  *pPTMpin[3] &= ~0x08; ---
Output2				 PM2      O     *pPTMpin[2] |= 0x04;  *pPTMpin[2] &= ~0x04; ---

Output1			     PE4      O    	PORTE |= 0x10;        PORTE &= ~0x10;       ---
4WD input            PE1      I     --    --  (PORTE & 0x02)       - on/off        

Spare Output2        PA0      O     PORTA |= 0x01;        PORTA &= ~0x01;       ---

	 
*/

// ----------------- INPUTS ------------------------------------
// -------------------------------------------------------------
 
// switchA
#define SWITCHA_ADC 0

// switchB
#define SWITCHB_ADC 1

// temperature
#define TEMPER_ADC 2

// switchC
#define SWITCHC_ADC 3

// line pressure
#define PRESSURE_ADC 4

// non-CAN MAP/TPS/MAF
#define NC_LOAD_ADC 5

// Voltage based shift buttons
#define VOLT_SHIFT_ADC 6

// Brake
#define BRAKE_ADC 7

// Paddle Down PAD06
#define PADDLE_DOWN PORTAD0
#define PADDLE_DOWNpin 0x40

// Paddle Up PT6
#define PADDLE_UP PTT
#define PADDLE_UPpin 0x40

// Brakes PAD07
#define BRAKE PORTAD0
#define BRAKEpin 0x80


// 4WD
#define FOUR_WD PORTE
#define FOUR_WDpin 0x02

  
// ------------------------ OUTPUTS ----------------------------
// -------------------------------------------------------------

// Output1
#define OUTPUT1port PORTE
#define OUTPUT1pin 0x10

// Output2
#define OUTPUT2port *pPTMpin[2]
#define OUTPUT2pin 0x04  

// Output3
#define OUTPUT3pwm PWMDTY1
#define OUTPUT3_ON PWMPER1
#define OUTPUT3_OFF 0

// Output4 
#define OUTPUT4port PORTA
#define OUTPUT4pin 0x01

// Output5 
#define OUTPUT5port PORTAD
#define OUTPUT5pin 0x02

// Output6
#define OUTPUT6port PORTAD
#define OUTPUT6pin 0x08

// Output7
#define OUTPUT7port *pPTTpin[7]
#define OUTPUT7pin 0x80

// Spare Output 1
#define SPARE1_OUTport *pPTTpin[7]
#define SPARE1pin 0x80

// Spare Output 2 
#define SPARE2_OUTport PORTA
#define SPARE2pin 0x01

// TCC Output
#define TCC_OUT PWMDTY3
#define TCC_ON PWMPER3
#define TCC_OFF 0

// Pressure Control Output
#define PC_DUTY PWMDTY2
#define PC_PER PWMPER2
#define PC_OFF 0 

// Speedo Output
#define SPEEDO_DTY PWMDTY4
#define SPEEDO_PER PWMPER4
#define SPEEDO_OFF 0

// SP0 Output
#define SP0_DTY PWMDTY4
#define SP0_PER PWMPER4
// #define SP0_OFF 0

// SP3 Output
#define SP3_DTY PWMDTY2
#define SP3_PER PWMPER2
// #define SP0_OFF 0

// LED1
#define LED1_OUT *pPTMpin[4]
#define LED1pin 0x10 

// LED2
#define LED2_OUT *pPTMpin[3]
#define LED2pin 0x08

// LED3
#define LED3_OUT *pPTMpin[5]
#define LED3pin 0x20

// LED4
#define LED4_OUT PORTB
#define LED4pin 0x10

  // The asterisk (*) in the above statements are placed in front of 'pointers' 
  // (variables that hold the memory address of other variables). The * is the 
  // 'indirection operator' - it says to take the value at the address of the 
  // variable pointed to.
  // For example, if A is a variable at address 0x00001000, and the value is 123, 
  // then a pointer to A could be set by
  // char A, *pA  <- declare the regular and pointer variable 
  // pA = &A      <- assign the address of A to the pointer pA
  // A = 123      <- assign the value of 123 to the variable A
  // then
  // pA = 0x00001000
  // and
  // *pA = 123
  // 
  // This is more than just two ways to do the same thing though. Only single values 
  // can be passed to C functions, and passing a pointer is one way around that to 
  // pass a array. 
  // That is, we pass the address of the start of the array (pA) and let the function 
  // increment that value to access the whole array.
