/******************* Set Up PWM on Timer Pins ******************************************

We want PWM on ports T1, T2, T3, & T4 (T0 is VSS input)
We want:
          Sol32:   T1 = PWM1 =  50 Hz = 0.0200 second period, variable duty cycle
          PC:      T2 = PWM2 = 293 Hz = 0.0034 second period, variable duty cycle
          TCC:     T3 = PWM3 =  50 Hz = 0.0200 second period, variable duty cycle 
                                                              (either 0% or 100% in this code, 
                                                              may be variable rate apply in 
                                                              future)
          speedout T4 = PWM4 = don't care - will use 0% or 100% only in this code
                                            (may use PWM in future for high speed signalling)         

1. Choose 8-bit mode (PWMCTL = 0x00 = default)  Could use 16 bit mode, but we don't need it.
2. Choose high polarity (PWMPOL = 0x1E for pins 1, 2, 3)
3. Choose left-aligned (PWMCAE = 0x00)
4. Select clock mode in PWMCLK = 0x1E:
     - PCLKn = 0 for 2N,
     - PCLKn = 1 for 2(N+1)  M.
5. Select N in PWMPRCLK register:
     - PCKA for channels 5, 4, 1, 0 (ISS, speedo, sol32, VSS);
     - PCKB for channels 7, 6, 3, 2 (spareoutput2, paddle up, TCC, PC).
6. If PCLKn = 1, select M = 1 to 255
     - PWMSCLA = M for channels 5, 4, 1, 0
     - PWMSCLB = M for channels 7, 6, 3, 2.
7. Select PWMPERn, normally between 100 and 255 (increased values give increased control resolution).
8. Enable desired PWM channels: PWME 1, 2, 3 and 4 for us so PWME |= 0x1E
9. Select PWMDTYn, normally between 0 and PWMPERn. Then
     Duty Cycle n = (PWMDTYn/PWMPERn)  100%
   Change duty cycle to control speed of motor or intensity of light, etc.
10. For 0% duty cycle, choose PWMDTYn = 0
*/
 
// Set up PWM outputs - PWM 1, 2, 3, & 4
// PT0,1,4,5 are on clkA - Sol 3/2 and speedo output
// PT2,3,6,7 are on clkB - PC and TCC
  if (in2ram.sp0_mode == 0) MODRR = 0x0E; else MODRR = 0x1E; // Make Port T pins 1, 2, & 3 be PWM (0x1E = 00001110), 
                   // PC and Output3 
                   // (PT3 is TCC, PT4 is spare port 0/speedo output,
                   // The MODRR register allows for mapping 
                   // of PWM channels to port T in the absence of port P 
                   // pins for the low pin count processors (like the 
                   // 48 pin MS-II MC9S12C64 processor).
  PWME     = 0x00; // disable PWMs initially  
  PWMPOL   = 0x1E; // polarity = 1, => go hi when start
 
// Set prescaler to 16. This divides bus clk (= PLLCLK/2 = 24 MHz)
// by 16. This gives 1.5 MHz timer, 1 tic = 2/3 usec.
// PWMPERx should be between 100 and 255, as it is 8 bit number 
// (100 gives us 1% increments, 255 is absolute maximum)
// We need to scale the clocks to get a 'tic' rate that works for both 
// the 50 and 293 Hertz requirements 

  PWMCLK   = 0x1E; // select scaled clocks SB, SA
  
  PWMPRCLK = 0x22; // prescale A,B clocks = bus/4 = 24/6 = 6 MHz
                   // 0x22 = 00100010 -> divide clock by 4

  PWMSCLA  = 0xFF; // PWM clk = SA clk/(2*SCLA) = 6MHz/510 = 0.085 msec clk
                   // for  50 Hertz, we will have 235 tic periods
                   
                              
  PWMSCLB  = 0x80; // PWM clk = SB clk/(2*SCLB) =  6MHz/256 = 0.0427 msec clk
                   // for 293 Hertz, we will have 80 tic periods
                   // can range from ~10 to 255 tics, or
                   // ~2340 Hertz to 92 Hertz
                      
  PWMCAE   = 0x00; // standard left align pulse:
                   //                            -----
                   //                           |     |______
                   //                             duty
                   //                            <--period--->

  TSCR2 |= 0x80;   // enable timer overflow interrupt (TOI)
  pTIC[0] = pTC0;
  ICintmask[0] = 0x01;
  pTOC[0] = pTC5;

  TCTL2 |= 0x88;   // bit OM1,3,5 = 1. OC output line high or 
	               // low iaw OL1,3,5 (not toggle or disable)
  TCTL1 |= 0x08;

  // set PWM period1 (usec) for solenoid C 
  PWMPER1 = (unsigned char) ((((long)inpram.OUT3_PWM_Pd*11765)+500000)/1000000);
                                                  // (period and duty cycle constant)
                                                  // 90% PWM% except 0% in first and 3/2 downshift
                                                  // 11.765 is (1/0.085) ticks/millisecond
                                                  // nominal value is 50 Hertz = 20 milliseconds 
                                                  // -> PWMPER1 = 20 * 11.765 = 235 tics
                                                  // 1000000 is because SOL32_PWM_Pd is in usec, not msec
                                                  //   *and* 11765 is supposed to be 11.765
                                                  // +500000 is for proper rounding (rather than truncate)
  
  // set PWM period2 for pressure control solenoid                                                       

  PWMPER2 = (unsigned char) ((((long)inpram.PC_PWM_Pd*23419)+500000)/1000000);
                                                  // (period is constant, duty cycle varies with load
                                                  // as set in pc_table[16x9])
                                                  // 23.419 is (1/0.0427) ticks/millisecond
                                                  // nominal value is 293 Hertz = 3.41 milliseconds 
                                                  // -> PWMPER2 = 3.41 * 23.419 = 80 tics
  
  // set PWM period3 for torque converter clutch                                   

  PWMPER3 = (unsigned char) ((((long)inpram.TCC_PWM_Pd*23419)+500000)/1000000);      
                                                  // (period is constant, DC varies from 0% to 100%)
                                                  // 23.419 is (1/0.0427) ticks/millisecond
                                                  // nominal value is 50 Hertz = 20 milliseconds 
                                                  // -> PWMPER2 = 3.41 * 23.419 = 80 tics
  
  // set PWM period4 (usec) for speedo output
  if (in2ram.sp0_mode != 0) // if not speedo mode
  	{
   	PWMPER4 = (unsigned char) ((((long)in2ram.sp0_per*11765)+500000)/1000000);
  	}
  // Initialize PWM Duty Cycles
  PWMDTY1 =    0;       // set Sol C PWM duty to 0%
  Output3(0); // set state indicator
  PWMDTY2 =    0;       // set PC PWM duty to 0%
  
  // if (inpram.cltch_enable == 2) inpram.TCC_PWM = 1;
 
  if (outpc.lock_TCC >= 1)
    {
    if (inpram.TCC_PWM == 1)  
      { // TCC PWM in use, set duty cycle to OFF state
      PWMDTY3 = (unsigned char) ((inpram.LUF_off*PWMPER3)/100);  // set PT3 to off state
      outpc.lock_TCC = tcc_unlock(); // set the flag for datalog
      }
      else // no TCC PWM
      {
      outpc.lock_TCC = tcc_unlock(); // set the flag for datalog
      }  // end  if (inpram.TCC_PWM == 1) 
    } // end if (outpc.lock_TCC == 0)

  PWMCNT1 = 0x00;   // clear counter (reset PWM)

  pTIC[1] = pTC0;
  ICintmask[1] = 0x01;