  // --- Set Up Timer Ports ---
  // Enable Timer Input Capture for PT0/PT5
  // --------------------------------------------------------------------------------
  // To use the Input Capture Function:
  //      - Enable the timer subsystem (set TEN bit of TSCR1)
  //      - Set the prescaler (TSCR2)
  // The 9S12 allows you to slow down the clock which drives the counter.
  // You can slow down the clock by dividing the 24 MHz clock by 2, 4, 8, 16, 32, 64 or 128.
  // You do this by writing to the prescaler bits (PR2:0) of the Timer System Control
  // Register 2 (TSCR2) Register at address 0x004D.
  //
  //   PR  Divide    Freq          Overflow Time
  //   000    1      24 MHz          2.7307 ms
  //   001    2      12 MHz          5.4613 ms
  //   010    4       6 MHz         10.9227 ms
  //   011    8       3 MHz         21.8453 ms
  //   100   16       1.5 MHz       43.6907 ms
  //   101   32       0.75 MHz      87.3813 ms
  //   110   64       0.375 MHz    174.7627 ms
  //   111  128       0.1875 MHz   349.5253 ms
  //
  // The clock ticks are counted in the variable TCNT (which is defined in the hsc12def.h file 
  // at memory location 0x0044). There are 0xFFFF 'tics' (65,536 counts).
  // In our case each 'tic' is 1/0.1875MHz  = 5.333 s
  //
  // To set up the timer:
  //  - Configure the processor to use a particular pin of PORTT for input
  //    capture
  //  - Configure the processor to set which edge (rising, falling, or either) you want to capture
  //  - Configure the processor to determine if you want an interrupt to be generated when the
  //    capture occurs
  
  TIOS = 0x1E;  // Input Capture or Output Compare Channel Configuration:
                //  - 0 The corresponding channel acts as an input capture
                //  - 1 The corresponding channel acts as an output compare
                // 0x1E = 00011110
                // Timer ch 0 (PT0/IOC0) = IC, 
                //       ch 5 (PT5/IOC5) = IC, 
                //       ch 1-4          = OC & PWM,
                //    - PT0 is VSS in input 
                //    - PT1 is solenoid C output (output3)
                //    - PT2 is PC output (PWM)
                //    - PT3 is TCC output  (is PWM in some cases)
                //    - PT4 is speedo output (is PWM but duty is set to zero or 100%)
                //    - PT5 is input shaft sensor input or non-CAN tach input
                //    - PT6 is paddle upshift input
                //    - PT7 is spare output1
                //    - used digital I/O
                // ch 6,7 = I/O output
  TSCR1 = 0x80; // Timer System Control Register 1, 10000000 = set port T0 to use timer 
                // (Timer Enable (TEN) bit = 1)
  TSCR2 = 0x07; // Timer System Control Register 12, 0x07 = 00000111
                // Set prescaler to divide by 128 in PR[0:2]
                // 349.53 millisecond overflow time, 5.333 sec period
                // for a 40 tooth input wheel, this is
                // 1/(0.34953*40) = 0.0715 rev/sec, less than 1/2 mile/hour
                // if teeth come is at 2 x 5.333 sec, this is about
                // 1/(0.00001066*40) = 2344 revs/sec, much faster than your car can go!
                // PR 	Divide 	Freq 		Overflow Rate
				// 000 	1 		24 MHz 		2.7307 ms
				// 001 	2 		12 MHz 		5.4613 ms
				// 010 	4 		6 MHz 		10.9227 ms
				// 011 	8 		3 MHz 		21.8453 ms
				// 100 	16 		1.5 MHz 	43.6907 ms
				// 101 	32 		0.75 MHz 	87.3813 ms
				// 110 	64 		0.375 MHz 	174.7627 ms
				// 111 	128 	0.1875 MHz 	349.5253 ms


  // Set the VSS Input Capture Edge
  if (inpram.InputCaptureEdge == 0) // InputCaptureEdge is for VSS
    {
     TCTL4 = 0x01;    // 0x01 = 00000001
    } 
    else // InputCaptureEdge == 1
    {
     TCTL4 = 0x02;    // 0x02 = 00000010
    }                 // VRx circuits are designed to trigger on rising edge
                      // Set edge to capture (EDGxB EDGxA of TCTL 3-4 regs)
                      // EDGxB EDGxA
                      //   0     0     Disabled
                      //   0     1     Rising Edge
                      //   1     0     Falling Edge
                      //   1     1     Either Edge
                      //
                      //   address  register bit7  bit6  bit5  bit4  bit3  bit2  bit1  bit0
                      //   0x004A    TCTL3   EDG7B EDG7A EDG6B EDG6A EDG5B EDG5A EDG4B EDG4A
                      //   0x004B    TCTL4   EDG4B EDG3A EDG2B EDG2A EDG1B EDG1A EDG0B EDG0A
                      // 0x02 = 00000010, ~0x04 = ~00000100 = 11111011

  // Set the ISS Input Capture Edge
  if ((inpram.ic2_usage & 0x03) > 0)  // if either option set for ic2 use
    {  
    if ((inpram.ic2_usage & 0x80) >> 7)  // if IC2 (ISS) capture edge set to falling
      {
      TCTL3 = 0x08;    // 00001000 = EDG5B = 1, EDG5A = 0 == falling edge
      } 
      else
      {
      TCTL3 = 0x04;    // 00000100 = EDG5B = 0, EDG5A = 1 == rising edge  
      } // end if ic2_usage & 0x80
    }
    else
    {
    TCTL3 = 0x00;    // clear the register - disable the interrupt
    } // end if ic2_usage & 0x03
                                 
  TFLG1 |= 0x21;       // 0x21 = 00100001 -> Clear IC0 Flag bits 0 (PT0) and 5 (PT5) in the 'main timer interrupt flag1' register
  TIE   |= 0x21;       // 0x21 = 00100001 -> Set Timer Interrupt Enable (TIE) on pin 0 (PT0) and pin 5 (PT5)
