/**************************************************************************
**
** SCI Communications
**
** Communications is established when the PC communications program sends
** a command character - the particular character sets the mode:
**
** "a" = send all of the realtime display variables (outpc structure) via tx port.
** "w"+<offset lsb>+<offset msb>+<nobytes>+<newbytes lsb, msb> = 
**    receive updated data parameter(s) and write into offset location
**    relative to start of data block, Can write to RAM only via serial (use 't'
**    below for Flash only tables) or write via CAN to Ram or Flash-only tables.
** "e" = same as "w" above, followed by "r" below to echo back value
** "r"+<offset lsb>+<offset msb>+<nobytes lsb, msb> = read and send back values
**     of a data parameter or block in table offset location.  Can be via serial 
**     or CAN. (Will read from RAM if Ram table exists, else will read Flash,) 
** "y" = verify inpram data block = inpflash data block, return no. bytes different.
** "b" = jump to flash burner routine and burn a ram data block into a flash 
**    data block.
** "t"+<tble_idx>+<tableWords of data> = receive new data for clt/mat/ego/maf 
**    FLASH tables via serial.
** "c" = Test communications - echo back Seconds
** "Q" = Send over Embedded Code Revision Number
** "S" = Send program title.
**
**************************************************************************/
void SerInit(void)  {
  
#define getCANid             40
#define getTableId           41
#define setBurningParameters 99

float ftmp;
uint32_t ultmp;

  SIU.PCR[89].R = 0x400;   // configure pad for TxDA function
  SIU.PCR[90].R = 0x400;   // configure pad for RxDA function
  
  ESCI_A.CR2.R = 0x2000;   // Module is enabled
#ifdef ECU_PPC5553
  // Set up SCI (rs232): SCI BR = Freq(=132MHz)/16/SBR
  // For BR = 115200, SBR = (8250000/115200) = 71.6 = 72;
  ftmp = (132000000.0f/(16.0f * (float)inpram->baud));
#endif
#ifdef ECU_PPC5634M
  // Set up SCI (rs232): SCI BR = Freq(=80MHz)/16/SBR
  // For BR = 115200, SBR = (5000000/115200) = 43.4 = 43;
  ftmp = (80000000.0f/(16.0f * (float)inpram->baud));
#endif
  ultmp = (uint32_t)ftmp;
  if((ftmp - (float)ultmp) > 0.5)ultmp++;
  ESCI_A.CR1.R = (ultmp << 16);
  ESCI_A.CR1.R &= ~0x00000088;   // xmit, xmtint disable
  ESCI_A.CR1.R |= 0x00000024;    // rcv, rcvint enable
  txcnt = 0;
  rxcnt = 0;
  txmode = 0;
  txgoal = 0;
  kill_ser = 0;
  kill_ser_t = 0xFFFF;
  vfy_flg = 0;
  burn_flag = 0;
  rcv_timeout = 0xFFFFFFFF;
  // set up interrupts
  INTC.PSR[146].R = 3;   // priority 1 lower than CAN
  ESCI_A.SR.R = 0x20000000;   // Clear the RDRF bit
  ESCI_A.SR.R = 0x80000000;   // Clear the TDRE bit -
           // need this or very 1st byte will be lost on PC side.
  return;
}

__declspec(interrupt nowarn SRR)
     void SCI_CommISR(void)
{
  int8_t dummy;
  static uint8_t tbleCnt;
  int16_t ix;
  static uint8_t rd_wr,CANid,sendCANdat=0,next_txmode;
  static int16_t xcntr;
  static uint16_t txptr,ntword,crxnbytes;
  static uint8_t tble_page[256];
  canmsg canmx;
  
  // if RDRF register not set, => transmit interrupt
  if(!ESCI_A.SR.B.RDRF)goto XMT_INT;

// Receive Interrupt
ESCI_A.SR.R = 0x20000000;   // Clear the RDRF bit
// Check if we are receiving new input parameter update
txgoal = 0;
rcv_timeout = 0xFFFFFFFF;
if(ESCI_A.SR.B.OR & 0x08000000)  {	 // check for rcv Overrun error
   txmode = 0;
   dummy = ESCI_A.DR.B.D;
   ESCI_A.CR1.R &= ~0x000000AC;   // rcv, xmt disable, interrupt disable
   kill_ser = 1;
   kill_ser_t = (uint16_t)(outpc.seconds + SER_TOUT);
   goto SER_DONE;
}

switch(txmode)  {

case 0:

  switch(ESCI_A.DR.B.D)  {
    case 'a':     // send back all real time ram output variables
  next_txmode = 1;
  txmode = getCANid;
  cksum = 0;
  break;

    case 'w':	  // receive new ram input data and write into offset location;
                  //  also used for forwarding CAN msgs bet. TS & gpio boards.
                  //  With this cmd in CAN pass-thru mode, 
  next_txmode = 5;
  txmode = getCANid;
  rd_wr = 1;
  break;

    case 'e':	  // same as 'w', but verify by echoing back values, up to 
                  //  sizeof(txbuf). 
  next_txmode = 5;// *** Don't use with CAN. ***
  txmode = getCANid;
  rd_wr = 2;
  break;

    case 'r':	  // read and send back ram input data from offset location;
                  //  also used for forwarding CAN msgs bet. TS & gpio boards,
  next_txmode = 5;// but *** no_bytes must be < sizeof(txbuf). ***
  txmode = getCANid;
  rd_wr = 0;
  cksum = 0;
  break;

    case 'y':      // Verify that a flash data block matches a
  next_txmode = 2; //  corresponding ram data block
  txmode = getCANid;
  break;
   	
    case 'b':     // burn a block of ram input values into flash;
                  //  also used for forwarding CAN msgs bet. TS & gpio boards
  next_txmode = setBurningParameters;
  txmode      = getCANid;
  break;

    case 't':     // update a flash table with following serial data
  txmode = 20;
  CANid = can_id; // not used in CAN pass-thru
  cksum = 0;
  break;

    case '!':     // start receiving reinit/reboot command
  txmode = 30;
  break;

    case 'c':     // send back seconds to test comms
  txcnt = 0;         
  txmode = 1;
  txgoal = 2;     // seconds is 1st 2 bytes of outpc structure
  tble_idx = 6;
  xcntr = 0;
  txbuf[0] = (int8_t)(outpc.seconds & 0x00FF);
  txbuf[1] = (int8_t)((outpc.seconds & 0xFF00) >> 4);
  ESCI_A.DR.B.D = txbuf[0];
  ESCI_A.CR1.R &= ~0x00000024;   // rcv, rcvint disable
  ESCI_A.CR1.R |= 0x00000088;    // xmit enable & xmit interrupt enable
  break;

    case 'Q':         // send code rev no.
  txcnt = 0;
  txmode = 4;
  txgoal = 20;
  ESCI_A.DR.B.D = RevNum[0]; 
  ESCI_A.CR1.R &= ~0x00000024;   // rcv, rcvint disable
  ESCI_A.CR1.R |= 0x00000088;    // xmit enable & xmit interrupt enable
  break;

    case 'S':         // send program title
  txcnt = 0;
  txmode = 5;
  txgoal = 32;
  ESCI_A.DR.B.D = Signature[0]; 
  ESCI_A.CR1.R &= ~0x00000024;   // rcv, rcvint disable
  ESCI_A.CR1.R |= 0x00000088;    // xmit enable & xmit interrupt enable
  break;
  
    case 'k':         // request checksum.
  txcnt = 0;
  txmode = 0;
  txgoal = 1;
  ESCI_A.DR.B.D = cksum; 
  ESCI_A.CR1.R &= ~0x00000024;   // rcv, rcvint disable
  ESCI_A.CR1.R |= 0x00000088;    // xmit enable & xmit interrupt enable
  break;

    default:
  break;
  }     // End of switch for received command
    break;

case getCANid: // Get CAN id for current command.
   CANid = ESCI_A.DR.B.D;
   if (CANid < MAX_CANBOARDS)
     txmode = getTableId;
   else	 {
  	 // CANid wrong - kill comms since don't know where to send data
     txmode = 0;
     ESCI_A.CR1.R &= ~0x000000AC;   // rcv, xmt disable, interrupt disable
     kill_ser = 1;
     kill_ser_t = (uint16_t)(outpc.seconds + SER_TOUT);
     goto SER_DONE;
   }
   break;
   
case getTableId: // Get table id for current command.
   tble_idx = ESCI_A.DR.B.D;
   if(CANid != can_id)  {
     txmode = next_txmode;
   } 
   else if(tble_idx >= NO_TBLES)  {
  	 // tble index wrong
     txmode = 0;
     //  kill comms since don't know which/ how much data
     ESCI_A.CR1.R &= ~0x000000AC;   // rcv, xmt disable, interrupt disable
     kill_ser = 1;
     kill_ser_t = (uint16_t)(outpc.seconds + SER_TOUT);
     goto SER_DONE;
   }
   else
     txmode = next_txmode;
   next_txmode = 0;
   if (txmode == 1) { 
     txcnt = 0;
     txgoal = sizeof(outpc);
     // load output variables into txbuf to avoid incoherent word data.
     // To work all words in structure must be word aligned, all longs
     // aligned on long boundaries.
     xcntr = 0;
     txptr = 0;
     *tableWordRam(tble_idx,0) = *((uint32_t *)&outpc);
     ESCI_A.DR.B.D = *tableByteRam(tble_idx, 0);
     cksum += ESCI_A.DR.B.D; 
     ESCI_A.CR1.R &= ~0x00000024;   // rcv, rcvint disable
     ESCI_A.CR1.R |= 0x00000088;    // xmit enable & xmit interrupt enable
   }
   else if (txmode == setBurningParameters) { // Burn command
     if(CANid != can_id)  {
  	   // set up single CAN message & forward to aux board
  	   canmx.msg_type = MSG_BURN; 
  	   canmx.destvarblk = tble_idx;
  	   canmx.destvaroff = 0;
  	   canmx.dest = CANid;
  	   canmx.varbyt = 0;		// no data bytes
  	   sendCanTx(&canmx);
     }
     else if(flocker == 0) { 
       flocker     = 0xCC;    	// set semaphore to prevent burning flash thru runaway code
       burn_flag   = 1;
       burn_idx = tble_idx;
     }
     txmode      = 0;			// handle burning in main loop
   } 
   else if(txmode == 2)  {
     vfy_flg = 1;
   	 vfy_fail = 0;
   }
   break;

case 5:
  	if(CANid != can_id)  {
  	  // TS wants to send data to aux board via CAN
  	  if(rd_wr >= 1)  {
  	    sendCANdat = CANid;
  	    rd_wr = 1;   // can't have echo in CAN
  	  } 
  	  // TS wants to request data from aux board via CAN
  	  else  {
  	    getCANdat = CANid;
  	  }
  	}
  	else  {          // serial only
  	  sendCANdat = 0;
  	  getCANdat = 0;
  	}
  	rxoffset = (uint16_t)(ESCI_A.DR.B.D << 8); // byte offset(msb) from start of inpram
  	txmode++;
  	break;

case 6:
  	rxoffset |= ESCI_A.DR.B.D;    // byte offset(lsb) from start of inpram
  	txmode++;
  	break;
  	
case 7:
  	rxnbytes = (uint16_t)(ESCI_A.DR.B.D << 8); 	// no. bytes (msb)
  	txmode++;
  	break;

case 8:
  	rxnbytes |= ESCI_A.DR.B.D;		    // no. bytes (lsb)
  	rxcnt = 0;
  	// check won't blow table arrays
  	if(CANid == can_id)  {
  	  if(rxoffset + rxnbytes > tableBytes(tble_idx))  {
  	    //  kill comms since don't know how much data being sent
  	    txmode = 0;
        ESCI_A.CR1.R &= ~0x000000AC;   // rcv, xmt disable, interrupt disable
  	    kill_ser = 1;
  	    kill_ser_t = (uint16_t)(outpc.seconds + SER_TOUT);
  	    goto SER_DONE;
  	  }
  	}
  	if(rd_wr == 0)  {		    // read & send back input data
  	  if(getCANdat)  {
  	    txmode = 0;         // done receiving all request data from TS
  	    // set up single CAN message & forward to aux board
  	    //   TS requesting data from aux board.
  	    //   Note: TS must only deal with 1 board at a time and wait til
  	    //   data back or timeout.
  	    canmx.msg_type = MSG_REQ; 
  	    canmx.destvarblk = tble_idx;
  	    canmx.myvarblk = 6;   // txbuf
  	    canmx.destvaroff = rxoffset;
  	    canmx.myvaroff = 0;
  	    canmx.dest = CANid;
  	    // rxnbytes must be <= 255
  	    canmx.varbyt = (uint8_t)rxnbytes;
  	    sendCanTx(&canmx);
  	    ltch_CAN = lmms + 5000;   // 0.5 sec timeout to get CAN data
  	    ctxboff = 0;
  	    ESCI_A.CR1.R &= ~0x00000024;   // rcv, rcvint disable
  	  } 
  	  else  {               // serial read only
  	    txcnt = 0;
  	    txmode = 3;
  	    txgoal = rxnbytes;
  	    if(tables[tble_idx].addrRam != MNULL)
  	      ESCI_A.DR.B.D = *tableByteRam(tble_idx, rxoffset);
  	    else  {
  	      ESCI_A.DR.B.D = *tableByteFlash(tble_idx, rxoffset);
  	    }
  	    cksum += ESCI_A.DR.B.D;
  	    // finish reading/sending rest in serial xmit mode
  	    ESCI_A.CR1.R &= ~0x00000024;   // rcv, rcvint disable
  	    ESCI_A.CR1.R |= 0x00000088;    // xmit enable & xmit interrupt enable
  	  }
  	}
  	else  {        // write data to input/ table data block or via CAN
  	  txmode++;
  	  crxnbytes = 0;
  	}
  	break;

case 9:
  	// Check for data overrun.  Note: this
  	// still not bullet proof because could write half the bytes
  	// then have overrun; need to store and write all at once if
  	// all is valid.
  	if(!ESCI_A.SR.B.OR)  {   // none yet- write data.
  	  if(sendCANdat)  {
  	    // want to buffer for 8 bytes for CAN pass thru
  	    txbuf[crxnbytes] = ESCI_A.DR.B.D;
  	  } 
  	  else  {   
  	    // write directly to RAM - don't need buffer since pc 
  	    //  data will be coherent
  	    *tableByteRam(tble_idx, rxoffset + rxcnt) = ESCI_A.DR.B.D;
  	  }
  	}
  	else  {
  	  txmode = 0;
  	  dummy = ESCI_A.DR.B.D;
  	  ESCI_A.CR1.R &= ~0x000000AC;   // rcv, xmt disable, interrupt disable
  	  kill_ser = 1;
  	  kill_ser_t = (uint16_t)(outpc.seconds + SER_TOUT);
  	  goto SER_DONE;
  	}
  	rxcnt++;
  	crxnbytes++;
  
  	if(sendCANdat)  {
  	  if((crxnbytes >= 8) || (rxcnt >= rxnbytes))  {
  	    // set up single CAN message & forward to aux board
  	    //   TS sending data to aux board
  	    // CAN faster than serial, so won't get overrun
  	    for(ix = 0; ix < crxnbytes; ix++)  {
  	      canmx.datbuf[ix] = txbuf[ix];
  	    }
  	    canmx.msg_type = MSG_CMD; 
  	    canmx.destvarblk = tble_idx;
  	    canmx.destvaroff = rxoffset;
  	    canmx.dest = sendCANdat;
  	    canmx.varbyt = (uint8_t)crxnbytes;
  	    sendCanTx(&canmx);
  	    rxoffset += crxnbytes;
  	    crxnbytes = 0;
  	  }
  	}

  	if(rxcnt >= rxnbytes)  {
  	  dummy = ESCI_A.DR.B.D;      // clear OR bit (read sci0sr1(done), sci0drl)
  	  if(rd_wr == 2)  {
  	    txcnt = 0;         // echo mode: send back data just written
  	    txmode = 3;        //	 for verification
  	    txgoal = rxnbytes;
  	    ESCI_A.DR.B.D = *tableByteRam(tble_idx, rxoffset);
  	    ESCI_A.CR1.R &= ~0x00000024;   // rcv, rcvint disable
  	    ESCI_A.CR1.R |= 0x00000088;    // xmit enable & xmit interrupt enable
  	  }
  	  else  {
  	    txmode = 0;         // done receiving/writing all the data
  	    sendCANdat = 0;
  	  } 
  	}       // finished all rxnbytes
  	break;

case 20:	 // d/load & burn tables - *** Do while engine NOT turning ***
  	// Note: this type table has no ram copy - its not intended for tuning
  	// Also, no pass-thru in this mode - must use w command for that.
  	tble_idx = ESCI_A.DR.B.D;    // type of table
  	if(tble_idx > NO_TBLES)	 {
  	  // tble index wrong - kill comms since don't know how much data being sent
  	  txmode = 0;
  	  ESCI_A.CR1.R &= ~0x000000AC;   // rcv, xmt disable, interrupt disable
  	  kill_ser = 1;
  	  kill_ser_t = (uint16_t)(outpc.seconds + SER_TOUT);
  	  goto SER_DONE;
  	}
  	ntword = 0;
	// erase table
  	Flash_Erase_Block(tableWordFlash(tble_idx, 0), tableBlk_no(tble_idx));
  	txmode++;
  	tbleCnt = 0;
  	break;

case 21:
  	cksum += ESCI_A.DR.B.D;
  	tble_page[tbleCnt] = ESCI_A.DR.B.D;    // get 255 bytes
  	if(tbleCnt >= 254)
  	  txmode++;
  	else
  	  tbleCnt++;
  	break;

case 22:
  cksum += ESCI_A.DR.B.D;
  tble_page[255] |= ESCI_A.DR.B.D;         // get 256th byte (= 1 page)
  // write table page
  Flash_Write_Page(tableWordFlash(tble_idx, ntword), (uint32_t *)tble_page);
  ntword += 8;
  if(ntword < tableWords(tble_idx))  {
    txmode = 21;
  	tbleCnt = 0;
  }
  else  {
    // done
    Flash_End_Write(tableBlk_no(tble_idx));
    txmode = 0;
  }
  break;

case 30:
  	if(ESCI_A.DR.B.D == '!')    // receive 2nd ! for reboot signal
  	  txmode++;
  	else if(ESCI_A.DR.B.D == 'x')  {  // received reinit signal (!x)
  	  reinit_flag = 1;
  	  txmode = 0;
  	}
  	else
  	  txmode = 0;
  	break;

case 31:
  	if(ESCI_A.DR.B.D == 'x')  {   // received complete reboot signal(!!x)
  	  // go to start of bootload pgm
  	  //asm (jmp $F800);
  	  //xxxxxxxxxxxxxx (void)reboot();
  	}
  	else if(ESCI_A.DR.B.D == '!')  {   // received complete reload signal(!!!)
  	  /*
  	  ESCI_A.CR1.R &= ~0x00000024;   // rcv, rcvint disable
  	  ESCI_A.CR1.R &= ~0x00000088;   // xmit, xmtint disable
  	  // xxxxxxx go to start of monitor code
  	  */
  	}
  	txmode = 0;
  	break;

default:
  dummy = ESCI_A.DR.B.D;   // dummy read of reg to clear int flag
  break;

  }     // End of case switch for received data
    
  if((txgoal == 0) && (txmode > 0))
    rcv_timeout = lmms + 5000;   // 0.5 sec timeout
  goto SER_DONE;

// Transmit Interrupt
XMT_INT:
  ESCI_A.SR.R = 0x80000000;   // Clear the TDRE bit
  txcnt++;
  if(txcnt < txgoal)  {
    if(txmode == 3)  {     // answer serial data request via ser data xmit
  	  if(tables[tble_idx].addrRam != MNULL)
  	    ESCI_A.DR.B.D = *tableByteRam(tble_idx, rxoffset + txcnt);
  	  else  {
  	    ESCI_A.DR.B.D = *tableByteFlash(tble_idx, rxoffset + txcnt);
  	  }
  	  if(rd_wr == 0)
  	    cksum += ESCI_A.DR.B.D;
    }
    else if(txmode == 4)  {
  	  ESCI_A.DR.B.D = RevNum[txcnt];
    }
    else if(txmode == 5)  {
  	  ESCI_A.DR.B.D = Signature[txcnt];
    }
    else if(txmode == 2)  {
  	  ESCI_A.DR.B.D = *((int8_t *)&vfy_fail + txcnt);
    } 
    else if(txmode == 6)  {
  	  // pass on CAN msg to TS via serial xmit
  	  // make sure serial not ahead of CAN
  	  if(txcnt < ctxboff)  {
  	    ESCI_A.DR.B.D = txbuf[txcnt];
  	    cksum += ESCI_A.DR.B.D;
  	  }
    } 
    else  {    // (txmode = 1) send back outpc data via serial 
      xcntr++;
      ESCI_A.DR.B.D = *tableByteRam(tble_idx, xcntr);
      cksum += ESCI_A.DR.B.D;
      if(xcntr >= 3)  {	 // xmitted 4 bytes, load 1 new 32-bit word
        xcntr = -1;
        txptr += 1;
        *tableWordRam(tble_idx,0) = *((uint32_t *)(&outpc) + txptr);
      }
    }
  } 
  else  {   // done transmitting
    if(txmode == 1)
      outpc.cksum = cksum;
    txcnt = 0;
    txgoal = 0;
    txmode = 0;
    ESCI_A.CR1.R &= ~0x00000088;    // xmit disable & xmit interrupt disable
    ESCI_A.CR1.R |= 0x00000024;     // rcv, rcvint re-enable
  }
SER_DONE:
  asm(mbar 0);   // done in sample epilogs to ensure interrupt flag cleared
  INTC.EOIR.R = 0;
  return;
}
